import unittest
from unittest import mock
from chia.server.outbound_message import NodeType

from chia_tea.chia_watchdog.api.FarmerHarvesterAPI import FarmerHarvesterAPI


from ..ChiaWatchdog import ChiaWatchdog
from ...utils.testing import async_test

from .update_from_farmer import update_from_farmer


class TestUpdatingFromFarmer(unittest.TestCase):
    def setUp(self) -> None:
        self.chia_config = {
            "self_hostname": "127.0.0.1",
            "farmer": {"rpc_port": 8008},
        }

    @async_test
    @mock.patch("chia_tea.chia_watchdog.api.update_from_farmer.FarmerRpcClient", autospec=True)
    @mock.patch("chia_tea.chia_watchdog.api.update_from_farmer.load_config", autospec=True)
    async def test_new_harvester_connected(self, load_config_mock, MockRpcClient):
        dog = ChiaWatchdog("", "")

        node_id = b"1n\x0f\xc4J\xb5q8\xc4\x98\x0b\xe7\\\xac\xd1\x82"
        plots = ["only", "length", "is", "used"]

        connection_result = {
            "bytes_read": 732920,
            "bytes_written": 736979,
            "creation_time": 1625781881.464225,
            "last_message_time": 1625856666.3932514,
            "local_port": 8447,
            "node_id": node_id,
            "peer_host": "127.0.0.1",
            "peer_port": 51844,
            "peer_server_port": 8448,
            "type": NodeType.HARVESTER.value,
        }
        harvester_result = {
            "success": True,
            "harvesters": [
                {"plots": plots, "connection": {"node_id": node_id.hex()}},
            ],
        }

        load_config_mock.return_value = self.chia_config
        MockRpcClient = MockRpcClient.create.return_value
        MockRpcClient.get_connections.return_value = [connection_result]
        MockRpcClient.get_harvesters.return_value = harvester_result
        MockRpcClient.close.return_value = None
        MockRpcClient.await_closed.return_value = None

        # do the thing
        await update_from_farmer(dog)

        # validate
        self.assertTrue(MockRpcClient.get_connections.called)
        self.assertTrue(MockRpcClient.get_harvesters.called)
        self.assertTrue(MockRpcClient.close.called)
        self.assertTrue(MockRpcClient.await_closed.called)

        harvester_list = dog.farmer_service.connections
        self.assertEqual(len(harvester_list), 1)

        harvester = harvester_list[0]
        for name, value in connection_result.items():
            self.assertEqual(
                value,
                getattr(harvester, name),
                "Harvester attribute '%s' does not match" % name,
            )
        self.assertEqual(harvester.n_plots, len(plots))

    @async_test
    @mock.patch("chia_tea.chia_watchdog.api.update_from_farmer.FarmerRpcClient", autospec=True)
    @mock.patch("chia_tea.chia_watchdog.api.update_from_farmer.load_config", autospec=True)
    async def test_harvester_disconnected(self, load_config_mock, MockRpcClient):
        dog = ChiaWatchdog("", "")

        node_id = b"1n\x0f\xc4J\xb5q8\xc4\x98\x0b\xe7\\\xac\xd1\x82"
        dog.farmer_service.connections = [
            FarmerHarvesterAPI(
                node_id=node_id,
                bytes_read=732920,
                bytes_written=736979,
                creation_time=1625781881.464225,
                last_message_time=1625856666.3932514,
                local_port=8447,
                peer_host="127.0.0.1",
                peer_port=51844,
                peer_server_port=8448,
                type=NodeType.HARVESTER.value,
            )
        ]
        harvester_result = {
            "success": True,
            "harvesters": [],
        }

        load_config_mock.return_value = self.chia_config
        MockRpcClient = MockRpcClient.create.return_value
        MockRpcClient.get_connections.return_value = []
        MockRpcClient.get_harvesters.return_value = harvester_result
        MockRpcClient.close.return_value = None
        MockRpcClient.await_closed.return_value = None

        # do the thing
        await update_from_farmer(dog)

        # validate
        self.assertTrue(MockRpcClient.get_connections.called)
        self.assertTrue(MockRpcClient.get_harvesters.called)
        self.assertTrue(MockRpcClient.close.called)
        self.assertTrue(MockRpcClient.await_closed.called)

        harvester_list = dog.farmer_service.connections
        self.assertEqual(len(harvester_list), 0)

    @async_test
    @mock.patch("chia_tea.chia_watchdog.api.update_from_farmer.FarmerRpcClient", autospec=True)
    @mock.patch("chia_tea.chia_watchdog.api.update_from_farmer.load_config", autospec=True)
    async def test_existing_harvester_is_updated(self, load_config_mock, MockRpcClient):
        dog = ChiaWatchdog("", "")

        node_id = b"1n\x0f\xc4J\xb5q8\xc4\x98\x0b\xe7\\\xac\xd1\x82"
        plots = ["only", "length", "is", "used"]
        old_harvester = FarmerHarvesterAPI(
            node_id=node_id,
            bytes_read=732920,
            bytes_written=736979,
            creation_time=1625781881.464225,
            last_message_time=1625856666.3932514,
            local_port=8447,
            peer_host="127.0.0.1",
            peer_port=51844,
            peer_server_port=8448,
            type=NodeType.HARVESTER.value,
            n_plots=len(plots),
        )
        dog.farmer_service.connections = [old_harvester]

        connection_result = {
            "bytes_read": 732921,
            "bytes_written": 736980,
            "creation_time": 1625781881.464226,
            "last_message_time": 1625856666.3932515,
            "local_port": 8448,
            "node_id": node_id,
            "peer_host": "127.0.0.2",
            "peer_port": 51845,
            "peer_server_port": 8449,
            "type": NodeType.HARVESTER.value,
        }
        harvester_result = {
            "success": True,
            "harvesters": [
                {"plots": plots, "connection": {"node_id": node_id.hex()}},
            ],
        }

        load_config_mock.return_value = self.chia_config
        MockRpcClient = MockRpcClient.create.return_value
        MockRpcClient.get_connections.return_value = [connection_result]
        MockRpcClient.get_harvesters.return_value = harvester_result
        MockRpcClient.close.return_value = None
        MockRpcClient.await_closed.return_value = None

        # do the thing
        await update_from_farmer(dog)

        # validate
        self.assertTrue(MockRpcClient.get_connections.called)
        self.assertTrue(MockRpcClient.get_harvesters.called)
        self.assertTrue(MockRpcClient.close.called)
        self.assertTrue(MockRpcClient.await_closed.called)
        self.assertTrue(load_config_mock.called)

        harvester_list = dog.farmer_service.connections
        self.assertEqual(len(harvester_list), 1)

        harvester = harvester_list[0]
        for name, value in connection_result.items():
            self.assertEqual(
                value,
                getattr(harvester, name),
                "Harvester attribute '%s' does not match" % name,
            )
        self.assertEqual(harvester.n_plots, len(plots))

    @async_test
    @mock.patch("chia_tea.chia_watchdog.api.update_from_farmer.FarmerRpcClient", autospec=True)
    @mock.patch("chia_tea.chia_watchdog.api.update_from_farmer.load_config", autospec=True)
    async def test_harvester_disconnects_between_two_api_calls(
        self, load_config_mock, MockRpcClient
    ):
        # pylint: disable=too-many-locals

        dog = ChiaWatchdog("", "")

        node_id1 = b"1n\x0f\xc4J\xb5q8\xc4\x98\x0b\xe7\\\xac\xd1\x82"
        plots1 = ["only", "length", "is", "used"]
        harvester1 = FarmerHarvesterAPI(
            node_id=node_id1,
            bytes_read=732920,
            bytes_written=736979,
            creation_time=1625781881.0,
            last_message_time=1625856666.0,
            local_port=8447,
            peer_host="127.0.0.1",
            peer_port=51844,
            peer_server_port=8448,
            type=NodeType.HARVESTER.value,
            n_plots=len(plots1),
        )
        node_id2 = b"1n\x0f\xc4J\xb5q8\xc4\x98\x0b\xe7\\\xac\xd1\x81"
        plots2 = ["only", "length", "is", "used", "too"]
        harvester2 = FarmerHarvesterAPI(
            node_id=node_id2,
            bytes_read=732921,
            bytes_written=736980,
            creation_time=1625781882.0,
            last_message_time=1625856667.0,
            local_port=8448,
            peer_host="127.0.0.2",
            peer_port=51845,
            peer_server_port=8449,
            type=NodeType.HARVESTER.value,
            n_plots=len(plots2),
        )
        dog.farmer_service.connections = [harvester1, harvester2]

        connection_result1 = {
            "bytes_read": harvester1.bytes_read,
            "bytes_written": harvester1.bytes_written,
            "creation_time": harvester1.creation_time,
            "last_message_time": harvester1.last_message_time,
            "local_port": harvester1.local_port,
            "node_id": harvester1.node_id,
            "peer_host": harvester1.peer_host,
            "peer_port": harvester1.peer_port,
            "peer_server_port": harvester1.peer_server_port,
            "type": NodeType.HARVESTER.value,
        }
        connection_result2 = {
            "bytes_read": harvester2.bytes_read,
            "bytes_written": harvester2.bytes_written,
            "creation_time": harvester2.creation_time,
            "last_message_time": harvester2.last_message_time,
            "local_port": harvester2.local_port,
            "node_id": harvester2.node_id,
            "peer_host": harvester2.peer_host,
            "peer_port": harvester2.peer_port,
            "peer_server_port": harvester2.peer_server_port,
            "type": NodeType.HARVESTER.value,
        }
        harvester_result = {
            "success": True,
            "harvesters": [
                {"plots": plots1, "connection": {"node_id": node_id1.hex()}},
                # harvester 2 is gone
            ],
        }

        load_config_mock.return_value = self.chia_config
        MockRpcClient = MockRpcClient.create.return_value
        MockRpcClient.get_connections.return_value = [connection_result1, connection_result2]
        MockRpcClient.get_harvesters.return_value = harvester_result
        MockRpcClient.close.return_value = None
        MockRpcClient.await_closed.return_value = None

        # do the thing
        await update_from_farmer(dog)

        # validate
        self.assertTrue(MockRpcClient.get_connections.called)
        self.assertTrue(MockRpcClient.get_harvesters.called)
        self.assertTrue(MockRpcClient.close.called)
        self.assertTrue(MockRpcClient.await_closed.called)
        self.assertTrue(load_config_mock.called)

        harvester_list = dog.farmer_service.connections
        self.assertEqual(len(harvester_list), 1)

        harvester = harvester_list[0]
        for name, value in connection_result1.items():
            self.assertEqual(
                value,
                getattr(harvester, name),
                "Harvester attribute '%s' does not match" % name,
            )
        self.assertEqual(harvester.n_plots, len(plots1))

    @async_test
    @mock.patch("chia_tea.chia_watchdog.api.update_from_farmer.FarmerRpcClient", autospec=True)
    @mock.patch("chia_tea.chia_watchdog.api.update_from_farmer.load_config", autospec=True)
    async def test_harvester_connects_between_two_api_calls(self, load_config_mock, MockRpcClient):
        dog = ChiaWatchdog("", "")

        node_id1 = b"1n\x0f\xc4J\xb5q8\xc4\x98\x0b\xe7\\\xac\xd1\x82"
        plots1 = ["only", "length", "is", "used"]
        harvester1 = FarmerHarvesterAPI(
            node_id=node_id1,
            bytes_read=732920,
            bytes_written=736979,
            creation_time=1625781881.0,
            last_message_time=1625856666.0,
            local_port=8447,
            peer_host="127.0.0.1",
            peer_port=51844,
            peer_server_port=8448,
            type=NodeType.HARVESTER.value,
            n_plots=len(plots1),
        )
        node_id2 = b"1n\x0f\xc4J\xb5q8\xc4\x98\x0b\xe7\\\xac\xd1\x81"
        plots2 = ["only", "length", "is", "used", "too"]
        dog.farmer_service.connections = [harvester1]

        connection_result1 = {
            "bytes_read": harvester1.bytes_read,
            "bytes_written": harvester1.bytes_written,
            "creation_time": harvester1.creation_time,
            "last_message_time": harvester1.last_message_time,
            "local_port": harvester1.local_port,
            "node_id": harvester1.node_id,
            "peer_host": harvester1.peer_host,
            "peer_port": harvester1.peer_port,
            "peer_server_port": harvester1.peer_server_port,
            "type": NodeType.HARVESTER.value,
        }
        harvester_result = {
            "success": True,
            "harvesters": [
                {"plots": plots1, "connection": {"node_id": node_id1.hex()}},
                {"plots": plots2, "connection": {"node_id": node_id2.hex()}},
                # harvester 2 is gone
            ],
        }

        load_config_mock.return_value = self.chia_config
        MockRpcClient = MockRpcClient.create.return_value
        MockRpcClient.get_connections.return_value = [connection_result1]
        MockRpcClient.get_harvesters.return_value = harvester_result
        MockRpcClient.close.return_value = None
        MockRpcClient.await_closed.return_value = None

        # do the thing
        await update_from_farmer(dog)

        # validate
        self.assertTrue(MockRpcClient.get_connections.called)
        self.assertTrue(MockRpcClient.get_harvesters.called)
        self.assertTrue(MockRpcClient.close.called)
        self.assertTrue(MockRpcClient.await_closed.called)
        self.assertTrue(load_config_mock.called)

        harvester_list = dog.farmer_service.connections
        self.assertEqual(len(harvester_list), 1)

        harvester = harvester_list[0]
        for name, value in connection_result1.items():
            self.assertEqual(
                value,
                getattr(harvester, name),
                "Harvester attribute '%s' does not match" % name,
            )
        self.assertEqual(harvester.n_plots, len(plots1))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chia_tea/protobuf/generated/monitoring_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from chia_tea.protobuf.generated import computer_info_pb2 as chia__tea_dot_protobuf_dot_generated_dot_computer__info__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='chia_tea/protobuf/generated/monitoring_service.proto',
  package='chia_tea.protobuf.generated.monitoring_service_pb2',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n4chia_tea/protobuf/generated/monitoring_service.proto\x12\x32\x63hia_tea.protobuf.generated.monitoring_service_pb2\x1a\x1bgoogle/protobuf/empty.proto\x1a/chia_tea/protobuf/generated/computer_info.proto\"%\n\x0fGetStateRequest\x12\x12\n\nmachine_id\x18\x01 \x01(\x03\"\x9c\x01\n\x11\x44\x61taUpdateRequest\x12\x12\n\nmachine_id\x18\x01 \x01(\x03\x12\x14\n\x0cmachine_name\x18\x04 \x01(\t\x12\x11\n\ttimestamp\x18\x02 \x01(\x01\x12J\n\x06\x65vents\x18\x03 \x03(\x0b\x32:.chia_tea.protobuf.generated.computer_info_pb2.UpdateEvent2\x9d\x02\n\nMonitoring\x12\x93\x01\n\x0fGetMachineState\x12\x43.chia_tea.protobuf.generated.monitoring_service_pb2.GetStateRequest\x1a;.chia_tea.protobuf.generated.computer_info_pb2.ComputerInfo\x12y\n\x14SendMonitoringUpdate\x12\x45.chia_tea.protobuf.generated.monitoring_service_pb2.DataUpdateRequest\x1a\x16.google.protobuf.Empty(\x01\x30\x01\x62\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,chia__tea_dot_protobuf_dot_generated_dot_computer__info__pb2.DESCRIPTOR,])




_GETSTATEREQUEST = _descriptor.Descriptor(
  name='GetStateRequest',
  full_name='chia_tea.protobuf.generated.monitoring_service_pb2.GetStateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='machine_id', full_name='chia_tea.protobuf.generated.monitoring_service_pb2.GetStateRequest.machine_id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=186,
  serialized_end=223,
)


_DATAUPDATEREQUEST = _descriptor.Descriptor(
  name='DataUpdateRequest',
  full_name='chia_tea.protobuf.generated.monitoring_service_pb2.DataUpdateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='machine_id', full_name='chia_tea.protobuf.generated.monitoring_service_pb2.DataUpdateRequest.machine_id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='machine_name', full_name='chia_tea.protobuf.generated.monitoring_service_pb2.DataUpdateRequest.machine_name', index=1,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='chia_tea.protobuf.generated.monitoring_service_pb2.DataUpdateRequest.timestamp', index=2,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='events', full_name='chia_tea.protobuf.generated.monitoring_service_pb2.DataUpdateRequest.events', index=3,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=226,
  serialized_end=382,
)

_DATAUPDATEREQUEST.fields_by_name['events'].message_type = chia__tea_dot_protobuf_dot_generated_dot_computer__info__pb2._UPDATEEVENT
DESCRIPTOR.message_types_by_name['GetStateRequest'] = _GETSTATEREQUEST
DESCRIPTOR.message_types_by_name['DataUpdateRequest'] = _DATAUPDATEREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetStateRequest = _reflection.GeneratedProtocolMessageType('GetStateRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETSTATEREQUEST,
  '__module__' : 'chia_tea.protobuf.generated.monitoring_service_pb2'
  # @@protoc_insertion_point(class_scope:chia_tea.protobuf.generated.monitoring_service_pb2.GetStateRequest)
  })
_sym_db.RegisterMessage(GetStateRequest)

DataUpdateRequest = _reflection.GeneratedProtocolMessageType('DataUpdateRequest', (_message.Message,), {
  'DESCRIPTOR' : _DATAUPDATEREQUEST,
  '__module__' : 'chia_tea.protobuf.generated.monitoring_service_pb2'
  # @@protoc_insertion_point(class_scope:chia_tea.protobuf.generated.monitoring_service_pb2.DataUpdateRequest)
  })
_sym_db.RegisterMessage(DataUpdateRequest)



_MONITORING = _descriptor.ServiceDescriptor(
  name='Monitoring',
  full_name='chia_tea.protobuf.generated.monitoring_service_pb2.Monitoring',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=385,
  serialized_end=670,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetMachineState',
    full_name='chia_tea.protobuf.generated.monitoring_service_pb2.Monitoring.GetMachineState',
    index=0,
    containing_service=None,
    input_type=_GETSTATEREQUEST,
    output_type=chia__tea_dot_protobuf_dot_generated_dot_computer__info__pb2._COMPUTERINFO,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SendMonitoringUpdate',
    full_name='chia_tea.protobuf.generated.monitoring_service_pb2.Monitoring.SendMonitoringUpdate',
    index=1,
    containing_service=None,
    input_type=_DATAUPDATEREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_MONITORING)

DESCRIPTOR.services_by_name['Monitoring'] = _MONITORING

# @@protoc_insertion_point(module_scope)



from chia.rpc.harvester_rpc_client import HarvesterRpcClient
from chia.util.config import load_config
from chia.util.default_root import DEFAULT_ROOT_PATH
from chia.util.ints import uint16

from ...utils.logger import log_runtime_async
from ..ChiaWatchdog import ChiaWatchdog
from .shared_settings import API_EXCEPTIONS


@log_runtime_async(__file__)
async def update_from_harvester(chia_dog: ChiaWatchdog):
    """Updates the chia dog with harvester data

    Parameters
    ----------
    chia_dog : ChiaWatchdog
        watchdog instance to be modified
    """

    try:
        config = load_config(DEFAULT_ROOT_PATH, "config.yaml", exit_on_error=False)
        self_hostname = config["self_hostname"]

        harvester_rpc_port = config["harvester"]["rpc_port"]
        harvester_client = await HarvesterRpcClient.create(
            self_hostname, uint16(harvester_rpc_port), DEFAULT_ROOT_PATH, config
        )

        plots_response = await harvester_client.get_plots()
        chia_dog.harvester_service.is_running = True
        if plots_response["success"]:
            chia_dog.harvester_service.plots = plots_response["plots"]
            chia_dog.harvester_service.failed_to_open_filenames = plots_response[
                "failed_to_open_filenames"
            ]
            chia_dog.harvester_service.not_found_filenames = plots_response["not_found_filenames"]

        chia_dog.harvester_service.plot_directories = await harvester_client.get_plot_directories()

    # pylint: disable=catching-non-exception
    except API_EXCEPTIONS:
        chia_dog.harvester_service.is_running = False
    finally:
        if "harvester_client" in locals():
            harvester_client.close()
            await harvester_client.await_closed()
        chia_dog.harvester_service.is_ready = True



import unittest

from chia_tea.protobuf.to_sqlite.generic import ProtoType, field_descriptor_is_list

from .generated.computer_info_pb2 import _COMPUTERINFO, _UPDATEEVENT
from .generated.config_pb2 import _MONITORINGCONFIG_CLIENTCONFIG_SENDUPDATEEVERY


class TestGenerated(unittest.TestCase):
    def test_send_update_every_matches_update_event_names(self):

        update_event_fields = set(
            field.name
            for field in _UPDATEEVENT.fields
            if field.type == ProtoType.MESSAGE.value
        )

        send_update_every_fields = set(
            field.name
            for field in _MONITORINGCONFIG_CLIENTCONFIG_SENDUPDATEEVERY.fields
        )

        missing_fields = update_event_fields - send_update_every_fields
        self.assertSetEqual(missing_fields, set())

        invalid_fields = send_update_every_fields - update_event_fields
        self.assertSetEqual(invalid_fields, set())

    def test_that_update_event_and_computer_info_have_same_field_types(self):

        update_event_message_types = set(
            field.message_type
            for field in _COMPUTERINFO.fields
            if field.message_type == ProtoType.MESSAGE.value
        )

        computer_info_message_types = set(
            field.message_type
            for field in _COMPUTERINFO.fields
            if field.message_type == ProtoType.MESSAGE.value
        )

        self.assertSetEqual(update_event_message_types, computer_info_message_types)

    def test_that_every_array_message_in_computer_info_has_an_id_entry(self):

        for field in _COMPUTERINFO.fields:

            if field.message_type != ProtoType.MESSAGE.value:
                continue

            submessage_type = field.message_type

            if field_descriptor_is_list(submessage_type):
                msg_field_names = list(submessage_type.fields_by_name)
                self.assertTrue(
                    "id" in msg_field_names,
                    "Message type '{0}' is a repeated field and must contain an entry 'id'.".format(
                        submessage_type.name
                    ),
                )



import unittest

from google.protobuf.json_format import ParseDict

from ...monitoring.MonitoringDatabase import MonitoringDatabase
from ..generated.computer_info_pb2 import ADD, DELETE, UPDATE, UpdateEvent
from .sql_cmds import get_update_events_from_db, insert_update_event_in_db


class TestSqlCmds(unittest.TestCase):
    def test_that_fetched_events_have_correct_event_type(self):

        event_types_to_test = [ADD, UPDATE, DELETE]

        db = MonitoringDatabase(":memory:")
        with db:
            update_event = ParseDict(
                js_dict=dict(
                    event_type=ADD,
                    farmer_harvester=dict(
                        id="some plot id 1",
                    ),
                ),
                message=UpdateEvent(),
            )
            for i_event, event_type in enumerate(event_types_to_test):
                update_event.event_type = event_type
                insert_update_event_in_db(
                    sql_cursor=db.cursor,
                    pb_message=update_event,
                    meta_attributes={
                        "machine_id": 1,
                        "timestamp": i_event,
                        "event_type": update_event.event_type,
                    },
                )

            machine_events = get_update_events_from_db(
                db.cursor,
                # timestamps inbetween which to fetch data
                0,
                len(event_types_to_test),
            )

            self.assertEqual(len(machine_events), 1)
            machine_id, event_list = next(iter(machine_events.items()))
            self.assertEqual(machine_id, 1)
            self.assertEqual(len(event_list), 3)
            for event, desired_event_type in zip(event_list, event_types_to_test):
                self.assertEqual(event.event_type, desired_event_type)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chia_tea/protobuf/generated/computer_info.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chia_tea.protobuf.generated import hardware_pb2 as chia__tea_dot_protobuf_dot_generated_dot_hardware__pb2
from chia_tea.protobuf.generated import chia_pb2 as chia__tea_dot_protobuf_dot_generated_dot_chia__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='chia_tea/protobuf/generated/computer_info.proto',
  package='chia_tea.protobuf.generated.computer_info_pb2',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n/chia_tea/protobuf/generated/computer_info.proto\x12-chia_tea.protobuf.generated.computer_info_pb2\x1a*chia_tea/protobuf/generated/hardware.proto\x1a&chia_tea/protobuf/generated/chia.proto\"\xe6\x05\n\x0c\x43omputerInfo\x12\x11\n\ttimestamp\x18\x01 \x01(\x01\x12\x12\n\nmachine_id\x18\x02 \x01(\x03\x12:\n\x03\x63pu\x18\x03 \x01(\x0b\x32-.chia_tea.protobuf.generated.hardware_pb2.Cpu\x12:\n\x03ram\x18\x04 \x01(\x0b\x32-.chia_tea.protobuf.generated.hardware_pb2.Ram\x12=\n\x05\x64isks\x18\x05 \x03(\x0b\x32..chia_tea.protobuf.generated.hardware_pb2.Disk\x12L\n\x0eplotting_plots\x18\x06 \x03(\x0b\x32\x34.chia_tea.protobuf.generated.chia_pb2.PlotInProgress\x12<\n\x06\x66\x61rmer\x18\x07 \x01(\x0b\x32,.chia_tea.protobuf.generated.chia_pb2.Farmer\x12Z\n\x11\x66\x61rmer_harvesters\x18\x08 \x03(\x0b\x32?.chia_tea.protobuf.generated.chia_pb2.HarvesterViewedFromFarmer\x12\x42\n\tharvester\x18\t \x01(\x0b\x32/.chia_tea.protobuf.generated.chia_pb2.Harvester\x12L\n\x0fharvester_plots\x18\n \x03(\x0b\x32\x33.chia_tea.protobuf.generated.chia_pb2.HarvesterPlot\x12<\n\x06wallet\x18\x0b \x01(\x0b\x32,.chia_tea.protobuf.generated.chia_pb2.Wallet\x12@\n\tprocesses\x18\x0c \x03(\x0b\x32-.chia_tea.protobuf.generated.chia_pb2.Process\"\xa8\x06\n\x0bUpdateEvent\x12L\n\nevent_type\x18\x03 \x01(\x0e\x32\x38.chia_tea.protobuf.generated.computer_info_pb2.EventType\x12<\n\x03\x63pu\x18\x04 \x01(\x0b\x32-.chia_tea.protobuf.generated.hardware_pb2.CpuH\x00\x12<\n\x03ram\x18\x05 \x01(\x0b\x32-.chia_tea.protobuf.generated.hardware_pb2.RamH\x00\x12>\n\x04\x64isk\x18\x06 \x01(\x0b\x32..chia_tea.protobuf.generated.hardware_pb2.DiskH\x00\x12>\n\x06\x66\x61rmer\x18\x07 \x01(\x0b\x32,.chia_tea.protobuf.generated.chia_pb2.FarmerH\x00\x12[\n\x10\x66\x61rmer_harvester\x18\x08 \x01(\x0b\x32?.chia_tea.protobuf.generated.chia_pb2.HarvesterViewedFromFarmerH\x00\x12M\n\x0eharvester_plot\x18\r \x01(\x0b\x32\x33.chia_tea.protobuf.generated.chia_pb2.HarvesterPlotH\x00\x12\x44\n\tharvester\x18\t \x01(\x0b\x32/.chia_tea.protobuf.generated.chia_pb2.HarvesterH\x00\x12>\n\x06wallet\x18\n \x01(\x0b\x32,.chia_tea.protobuf.generated.chia_pb2.WalletH\x00\x12@\n\x07process\x18\x0b \x01(\x0b\x32-.chia_tea.protobuf.generated.chia_pb2.ProcessH\x00\x12M\n\rplotting_plot\x18\x0c \x01(\x0b\x32\x34.chia_tea.protobuf.generated.chia_pb2.PlotInProgressH\x00\x42\x0c\n\nevent_data*6\n\tEventType\x12\x08\n\x04NONE\x10\x00\x12\x07\n\x03\x41\x44\x44\x10\x01\x12\n\n\x06UPDATE\x10\x02\x12\n\n\x06\x44\x45LETE\x10\x03\x62\x06proto3'
  ,
  dependencies=[chia__tea_dot_protobuf_dot_generated_dot_hardware__pb2.DESCRIPTOR,chia__tea_dot_protobuf_dot_generated_dot_chia__pb2.DESCRIPTOR,])

_EVENTTYPE = _descriptor.EnumDescriptor(
  name='EventType',
  full_name='chia_tea.protobuf.generated.computer_info_pb2.EventType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NONE', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ADD', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UPDATE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DELETE', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1738,
  serialized_end=1792,
)
_sym_db.RegisterEnumDescriptor(_EVENTTYPE)

EventType = enum_type_wrapper.EnumTypeWrapper(_EVENTTYPE)
NONE = 0
ADD = 1
UPDATE = 2
DELETE = 3



_COMPUTERINFO = _descriptor.Descriptor(
  name='ComputerInfo',
  full_name='chia_tea.protobuf.generated.computer_info_pb2.ComputerInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='chia_tea.protobuf.generated.computer_info_pb2.ComputerInfo.timestamp', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='machine_id', full_name='chia_tea.protobuf.generated.computer_info_pb2.ComputerInfo.machine_id', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cpu', full_name='chia_tea.protobuf.generated.computer_info_pb2.ComputerInfo.cpu', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ram', full_name='chia_tea.protobuf.generated.computer_info_pb2.ComputerInfo.ram', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='disks', full_name='chia_tea.protobuf.generated.computer_info_pb2.ComputerInfo.disks', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='plotting_plots', full_name='chia_tea.protobuf.generated.computer_info_pb2.ComputerInfo.plotting_plots', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='farmer', full_name='chia_tea.protobuf.generated.computer_info_pb2.ComputerInfo.farmer', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='farmer_harvesters', full_name='chia_tea.protobuf.generated.computer_info_pb2.ComputerInfo.farmer_harvesters', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='harvester', full_name='chia_tea.protobuf.generated.computer_info_pb2.ComputerInfo.harvester', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='harvester_plots', full_name='chia_tea.protobuf.generated.computer_info_pb2.ComputerInfo.harvester_plots', index=9,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='wallet', full_name='chia_tea.protobuf.generated.computer_info_pb2.ComputerInfo.wallet', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='processes', full_name='chia_tea.protobuf.generated.computer_info_pb2.ComputerInfo.processes', index=11,
      number=12, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=183,
  serialized_end=925,
)


_UPDATEEVENT = _descriptor.Descriptor(
  name='UpdateEvent',
  full_name='chia_tea.protobuf.generated.computer_info_pb2.UpdateEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='event_type', full_name='chia_tea.protobuf.generated.computer_info_pb2.UpdateEvent.event_type', index=0,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cpu', full_name='chia_tea.protobuf.generated.computer_info_pb2.UpdateEvent.cpu', index=1,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ram', full_name='chia_tea.protobuf.generated.computer_info_pb2.UpdateEvent.ram', index=2,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='disk', full_name='chia_tea.protobuf.generated.computer_info_pb2.UpdateEvent.disk', index=3,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='farmer', full_name='chia_tea.protobuf.generated.computer_info_pb2.UpdateEvent.farmer', index=4,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='farmer_harvester', full_name='chia_tea.protobuf.generated.computer_info_pb2.UpdateEvent.farmer_harvester', index=5,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='harvester_plot', full_name='chia_tea.protobuf.generated.computer_info_pb2.UpdateEvent.harvester_plot', index=6,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='harvester', full_name='chia_tea.protobuf.generated.computer_info_pb2.UpdateEvent.harvester', index=7,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='wallet', full_name='chia_tea.protobuf.generated.computer_info_pb2.UpdateEvent.wallet', index=8,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='process', full_name='chia_tea.protobuf.generated.computer_info_pb2.UpdateEvent.process', index=9,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='plotting_plot', full_name='chia_tea.protobuf.generated.computer_info_pb2.UpdateEvent.plotting_plot', index=10,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='event_data', full_name='chia_tea.protobuf.generated.computer_info_pb2.UpdateEvent.event_data',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=928,
  serialized_end=1736,
)

_COMPUTERINFO.fields_by_name['cpu'].message_type = chia__tea_dot_protobuf_dot_generated_dot_hardware__pb2._CPU
_COMPUTERINFO.fields_by_name['ram'].message_type = chia__tea_dot_protobuf_dot_generated_dot_hardware__pb2._RAM
_COMPUTERINFO.fields_by_name['disks'].message_type = chia__tea_dot_protobuf_dot_generated_dot_hardware__pb2._DISK
_COMPUTERINFO.fields_by_name['plotting_plots'].message_type = chia__tea_dot_protobuf_dot_generated_dot_chia__pb2._PLOTINPROGRESS
_COMPUTERINFO.fields_by_name['farmer'].message_type = chia__tea_dot_protobuf_dot_generated_dot_chia__pb2._FARMER
_COMPUTERINFO.fields_by_name['farmer_harvesters'].message_type = chia__tea_dot_protobuf_dot_generated_dot_chia__pb2._HARVESTERVIEWEDFROMFARMER
_COMPUTERINFO.fields_by_name['harvester'].message_type = chia__tea_dot_protobuf_dot_generated_dot_chia__pb2._HARVESTER
_COMPUTERINFO.fields_by_name['harvester_plots'].message_type = chia__tea_dot_protobuf_dot_generated_dot_chia__pb2._HARVESTERPLOT
_COMPUTERINFO.fields_by_name['wallet'].message_type = chia__tea_dot_protobuf_dot_generated_dot_chia__pb2._WALLET
_COMPUTERINFO.fields_by_name['processes'].message_type = chia__tea_dot_protobuf_dot_generated_dot_chia__pb2._PROCESS
_UPDATEEVENT.fields_by_name['event_type'].enum_type = _EVENTTYPE
_UPDATEEVENT.fields_by_name['cpu'].message_type = chia__tea_dot_protobuf_dot_generated_dot_hardware__pb2._CPU
_UPDATEEVENT.fields_by_name['ram'].message_type = chia__tea_dot_protobuf_dot_generated_dot_hardware__pb2._RAM
_UPDATEEVENT.fields_by_name['disk'].message_type = chia__tea_dot_protobuf_dot_generated_dot_hardware__pb2._DISK
_UPDATEEVENT.fields_by_name['farmer'].message_type = chia__tea_dot_protobuf_dot_generated_dot_chia__pb2._FARMER
_UPDATEEVENT.fields_by_name['farmer_harvester'].message_type = chia__tea_dot_protobuf_dot_generated_dot_chia__pb2._HARVESTERVIEWEDFROMFARMER
_UPDATEEVENT.fields_by_name['harvester_plot'].message_type = chia__tea_dot_protobuf_dot_generated_dot_chia__pb2._HARVESTERPLOT
_UPDATEEVENT.fields_by_name['harvester'].message_type = chia__tea_dot_protobuf_dot_generated_dot_chia__pb2._HARVESTER
_UPDATEEVENT.fields_by_name['wallet'].message_type = chia__tea_dot_protobuf_dot_generated_dot_chia__pb2._WALLET
_UPDATEEVENT.fields_by_name['process'].message_type = chia__tea_dot_protobuf_dot_generated_dot_chia__pb2._PROCESS
_UPDATEEVENT.fields_by_name['plotting_plot'].message_type = chia__tea_dot_protobuf_dot_generated_dot_chia__pb2._PLOTINPROGRESS
_UPDATEEVENT.oneofs_by_name['event_data'].fields.append(
  _UPDATEEVENT.fields_by_name['cpu'])
_UPDATEEVENT.fields_by_name['cpu'].containing_oneof = _UPDATEEVENT.oneofs_by_name['event_data']
_UPDATEEVENT.oneofs_by_name['event_data'].fields.append(
  _UPDATEEVENT.fields_by_name['ram'])
_UPDATEEVENT.fields_by_name['ram'].containing_oneof = _UPDATEEVENT.oneofs_by_name['event_data']
_UPDATEEVENT.oneofs_by_name['event_data'].fields.append(
  _UPDATEEVENT.fields_by_name['disk'])
_UPDATEEVENT.fields_by_name['disk'].containing_oneof = _UPDATEEVENT.oneofs_by_name['event_data']
_UPDATEEVENT.oneofs_by_name['event_data'].fields.append(
  _UPDATEEVENT.fields_by_name['farmer'])
_UPDATEEVENT.fields_by_name['farmer'].containing_oneof = _UPDATEEVENT.oneofs_by_name['event_data']
_UPDATEEVENT.oneofs_by_name['event_data'].fields.append(
  _UPDATEEVENT.fields_by_name['farmer_harvester'])
_UPDATEEVENT.fields_by_name['farmer_harvester'].containing_oneof = _UPDATEEVENT.oneofs_by_name['event_data']
_UPDATEEVENT.oneofs_by_name['event_data'].fields.append(
  _UPDATEEVENT.fields_by_name['harvester_plot'])
_UPDATEEVENT.fields_by_name['harvester_plot'].containing_oneof = _UPDATEEVENT.oneofs_by_name['event_data']
_UPDATEEVENT.oneofs_by_name['event_data'].fields.append(
  _UPDATEEVENT.fields_by_name['harvester'])
_UPDATEEVENT.fields_by_name['harvester'].containing_oneof = _UPDATEEVENT.oneofs_by_name['event_data']
_UPDATEEVENT.oneofs_by_name['event_data'].fields.append(
  _UPDATEEVENT.fields_by_name['wallet'])
_UPDATEEVENT.fields_by_name['wallet'].containing_oneof = _UPDATEEVENT.oneofs_by_name['event_data']
_UPDATEEVENT.oneofs_by_name['event_data'].fields.append(
  _UPDATEEVENT.fields_by_name['process'])
_UPDATEEVENT.fields_by_name['process'].containing_oneof = _UPDATEEVENT.oneofs_by_name['event_data']
_UPDATEEVENT.oneofs_by_name['event_data'].fields.append(
  _UPDATEEVENT.fields_by_name['plotting_plot'])
_UPDATEEVENT.fields_by_name['plotting_plot'].containing_oneof = _UPDATEEVENT.oneofs_by_name['event_data']
DESCRIPTOR.message_types_by_name['ComputerInfo'] = _COMPUTERINFO
DESCRIPTOR.message_types_by_name['UpdateEvent'] = _UPDATEEVENT
DESCRIPTOR.enum_types_by_name['EventType'] = _EVENTTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ComputerInfo = _reflection.GeneratedProtocolMessageType('ComputerInfo', (_message.Message,), {
  'DESCRIPTOR' : _COMPUTERINFO,
  '__module__' : 'chia_tea.protobuf.generated.computer_info_pb2'
  # @@protoc_insertion_point(class_scope:chia_tea.protobuf.generated.computer_info_pb2.ComputerInfo)
  })
_sym_db.RegisterMessage(ComputerInfo)

UpdateEvent = _reflection.GeneratedProtocolMessageType('UpdateEvent', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEEVENT,
  '__module__' : 'chia_tea.protobuf.generated.computer_info_pb2'
  # @@protoc_insertion_point(class_scope:chia_tea.protobuf.generated.computer_info_pb2.UpdateEvent)
  })
_sym_db.RegisterMessage(UpdateEvent)


# @@protoc_insertion_point(module_scope)



import sqlite3
from io import StringIO
from typing import Iterable, List

from rich.console import Console
from rich.table import Table

from ..common import catch_errors_as_message, open_database_read_only

ROW_LIMIT = 5


async def format_sql_rows(rows: list, names: Iterable[str]) -> str:
    """Format the sql rows as a table

    Parameters
    ----------
    rows : list
        row values as list
    names : Iterable[str]
        names of the columns

    Returns
    -------
    msg : str
        msg to print
    """
    msg = ""

    table = Table()

    n_rows = min(len(rows), ROW_LIMIT)
    if not n_rows:
        return msg

    # format table
    console = Console(file=StringIO())
    # column names
    for name in names:
        table.add_column(name)
    # row values
    for i_row, row in enumerate(rows):
        if i_row > n_rows:
            break
        table.add_row(*tuple(str(value) for value in row))
    console.print(table)

    # pylint: disable=no-member
    output = console.file.getvalue()
    msg += f"```\n{output}```"

    if len(rows) > ROW_LIMIT:
        msg += f"\n⚠️ Displaying only {ROW_LIMIT} of {len(rows)} entries."

    return msg


@catch_errors_as_message
async def sql_cmd(db_filepath: str, cmd: str) -> List[str]:
    """Execute an sql command and return the results as text

    Parameters
    ----------
    db_filepath : str
        filepath to the database
    cmds : str
        sql command

    Returns
    -------
    messages : List[str]
        list of messages to print
    """

    messages = []

    with open_database_read_only(db_filepath) as cursor:
        sql_cursor: sqlite3.Cursor = cursor
        try:
            sql_cursor.execute(cmd)
            rows = sql_cursor.fetchall()

            # this contains the names but if nothing is
            # found this is none
            description = sql_cursor.description or tuple()
            entry_names = tuple(entry[0] for entry in description)

            msg = await format_sql_rows(rows, entry_names)
            if msg:
                messages.append(msg)

        except sqlite3.OperationalError as err:
            messages.append(str(err))

    if not messages:
        messages.append("😐 No entries found ")

    return messages



import unittest
from datetime import datetime

from ..protobuf.generated.chia_pb2 import PlotInProgress
from .formatting import plot_in_progress_pb2_as_markdown


class TestFormatting(unittest.TestCase):
    def test_plot_in_progress_pb2_as_markdown(self):

        plot_in_progress = PlotInProgress(
            id="some-id",
            pool_public_key="some-pool-key",
            start_time=datetime.now().timestamp(),
            state="Phase1",
            progress=0.1,
        )
        output = plot_in_progress_pb2_as_markdown(plot_in_progress=plot_in_progress)

        self.assertIn("Plot some-id", output)
        self.assertIn("Since: ", output)
        self.assertIn("State: Phase1", output)
        self.assertIn("Progress: 10.0%", output)



import os
import platform
from typing import List

import cpuinfo
import psutil

from ...protobuf.generated.hardware_pb2 import Cpu, Disk, Ram
from ...utils.logger import log_runtime_async


def __get_cpu_temp() -> float:
    """Get the current cpu temperature

    Returns
    -------
    temp : float
        cpu temperature
    """
    temp = 0

    system_string = platform.system()

    # Linux
    if system_string == "Linux":
        temperatures = psutil.sensors_temperatures(fahrenheit=False)

        # cpu_thermal = raspberrypi
        # coretemp = common on linux
        cpu_temp_list = temperatures.get("cpu_thermal") or temperatures.get("coretemp")
        if cpu_temp_list:
            temp_sum = sum(core_temp.current for core_temp in cpu_temp_list)
            temp = temp_sum / len(cpu_temp_list)
    # Mac
    elif system_string == "Darwin":
        pass
    # Windows
    elif system_string == "Windows":
        pass

    return temp


CPU_NAME = processor_name = (
    cpuinfo.get_cpu_info().get("brand_raw") or platform.processor()
)


@log_runtime_async(__file__)
async def collect_cpu_info() -> Cpu:
    """Collect all info about the CPU

    Returns
    -------
    ram_info : Cpu
        info about the CPU
    """

    return Cpu(
        clock_speed=psutil.cpu_freq().current,
        usage=psutil.cpu_percent(),
        temperature=__get_cpu_temp(),
        name=CPU_NAME,
        n_vcores=os.cpu_count(),
    )


@log_runtime_async(__file__)
async def collect_ram_info() -> Ram:
    """Collect all info about the RAM

    Returns
    -------
    ram_info : Ram
        info about the RAM
    """
    swap_memory = psutil.swap_memory()
    virtual_memory = psutil.virtual_memory()

    return Ram(
        total_ram=virtual_memory.total,
        used_ram=virtual_memory.used,
        total_swap=swap_memory.total,
        used_swap=swap_memory.used,
    )


@log_runtime_async(__file__)
async def collect_disk_info() -> List[Disk]:
    """Collect all about the disks

    Returns
    -------
    disk_info_list : List[Disk]
        list with info about different disks
    """
    disk_info_list = []

    drives_to_skip = [
        "/dev/loop",
        "/boot",
        "/etc",
        "/snap",
    ]

    disk_partitions = psutil.disk_partitions()
    for partition in disk_partitions:
        if any(
            partition.mountpoint.startswith(drive_name) for drive_name in drives_to_skip
        ):
            continue

        disk_usage = psutil.disk_usage(partition.mountpoint)

        disk_info_list.append(
            Disk(
                id=partition.mountpoint,
                total_space=disk_usage.total,
                used_space=disk_usage.used,
                device=partition.device,
                mountpoint=partition.mountpoint,
                fstype=partition.fstype,
                mount_options=partition.opts,
            )
        )

    return disk_info_list



import os
import tempfile
import unittest

from google.protobuf.json_format import ParseDict

from ...monitoring.MonitoringDatabase import MonitoringDatabase
from ...protobuf.generated.computer_info_pb2 import ADD, UpdateEvent
from ...protobuf.generated.monitoring_service_pb2 import DataUpdateRequest
from ...utils.testing import async_test
from .harvesters import harvesters_cmd


class TestHarvestersCmd(unittest.TestCase):
    @async_test
    async def test_no_harvesters_case(self) -> None:

        with tempfile.TemporaryDirectory() as tmpdir:
            db_filepath = os.path.join(tmpdir, "temp.db")

            with MonitoringDatabase(db_filepath):
                messages = await harvesters_cmd(db_filepath)

                self.assertEqual(len(messages), 1)
                self.assertTrue(messages[0].startswith("No Harvesters"))

    @async_test
    async def test_not_running_harvester_not_displayed(self) -> None:

        with tempfile.TemporaryDirectory() as tmpdir:
            db_filepath = os.path.join(tmpdir, "tmp.db")

            with MonitoringDatabase(db_filepath) as db:
                update_event = ParseDict(
                    js_dict=dict(
                        event_type=ADD,
                        harvester=dict(
                            is_running=False,
                        ),
                    ),
                    message=UpdateEvent(),
                )
                request = DataUpdateRequest(
                    machine_id=2,
                    machine_name="machine B",
                    timestamp=1000,
                    events=[update_event],
                )
                db.store_data_update_request(request)

                messages = await harvesters_cmd(db_filepath)

                self.assertEqual(len(messages), 1)
                self.assertTrue(messages[0].startswith("No Harvesters"))
                self.assertTrue("machine B" not in messages[0])

    @async_test
    async def test_harvesters_in_db_case(self) -> None:

        with tempfile.TemporaryDirectory() as tmpdir:
            db_filepath = os.path.join(tmpdir, "tmp.db")

            with MonitoringDatabase(db_filepath) as db:
                # machine A:
                # - Harvester (running)
                # - 1 Disk
                # - 1 Plot
                update_events = [
                    ParseDict(
                        js_dict=dict(
                            event_type=ADD,
                            harvester=dict(
                                is_running=True,
                                n_proofs=1,
                            ),
                        ),
                        message=UpdateEvent(),
                    ),
                    ParseDict(
                        js_dict=dict(
                            event_type=ADD,
                            harvester_plot=dict(
                                id="plot1",
                                filename="/path/to/plot1",
                                filesize=1e8,
                                pool_contract_puzzle_hash="pool_contract_puzzle_hash",
                                pool_public_key="pool_public_key",
                                size=32,
                                time_modified=10,
                            ),
                        ),
                        message=UpdateEvent(),
                    ),
                    ParseDict(
                        js_dict=dict(
                            event_type=ADD,
                            harvester_plot=dict(
                                id="plot1",
                                filename="/path/to/plot1",
                                filesize=1e8,
                                pool_contract_puzzle_hash="pool_contract_puzzle_hash",
                                pool_public_key="pool_public_key",
                                size=32,
                                time_modified=10,
                            ),
                        ),
                        message=UpdateEvent(),
                    ),
                    ParseDict(
                        js_dict=dict(
                            event_type=ADD,
                            disk=dict(
                                id="disk1",
                                name="some disk",
                                total_space=2e8,
                                used_space=1e8,
                                device="some device",
                                mountpoint="/mnt/plots",
                                fstype="ext4",
                                mount_options="ro",
                            ),
                        ),
                        message=UpdateEvent(),
                    ),
                ]
                request = DataUpdateRequest(
                    machine_id=1,
                    machine_name="machine A",
                    timestamp=1000,
                    events=update_events,
                )
                db.store_data_update_request(request)

                messages = await harvesters_cmd(db_filepath)

                # no failure
                self.assertEqual(len(messages), 1)
                msg = messages[0]
                self.assertFalse(msg.startswith("Traceback"))
                # display online harvester
                self.assertTrue("machine A" in msg)
                self.assertEqual(msg.count("🚜 Harvester"), 1)
                self.assertEqual(msg.count("disk1"), 1)
                self.assertEqual(msg.count("plots: 1"), 1)
                self.assertEqual(msg.count("proofs: 1"), 1)



import os
import tempfile
import unittest

from google.protobuf.json_format import ParseDict

from ...monitoring.MonitoringDatabase import MonitoringDatabase
from ...protobuf.generated.computer_info_pb2 import ADD, UpdateEvent
from ...protobuf.generated.monitoring_service_pb2 import DataUpdateRequest
from ...utils.testing import async_test
from .sql import sql_cmd


class TestSqlCmd(unittest.TestCase):
    @async_test
    async def test_syntax_error(self) -> None:
        with tempfile.TemporaryDirectory() as tmpdir:
            db_filepath = os.path.join(tmpdir, "temp.db")
            with MonitoringDatabase(db_filepath):
                messages = await sql_cmd(db_filepath, "ERROR ~!!#*)!(*#;")

                self.assertEqual(len(messages), 1)
                self.assertIn("syntax error", messages[0])

    @async_test
    async def test_empty_command(self) -> None:
        with tempfile.TemporaryDirectory() as tmpdir:
            db_filepath = os.path.join(tmpdir, "temp.db")
            with MonitoringDatabase(db_filepath):
                messages = await sql_cmd(
                    db_filepath,
                    "",
                )
                self.assertEqual(len(messages), 1)
                self.assertIn("No entries found", messages[0])

    @async_test
    async def test_if_readonly_protected(self) -> None:
        with tempfile.TemporaryDirectory() as tmpdir:
            db_filepath = os.path.join(tmpdir, "temp.db")
            with MonitoringDatabase(db_filepath):
                messages = await sql_cmd(
                    db_filepath,
                    "CREATE TABLE yay (pewpew text)",
                )
                self.assertEqual(len(messages), 1)
                self.assertEqual("attempt to write a readonly database", messages[0])

    @async_test
    async def test_valid_select_cmd(self) -> None:
        with tempfile.TemporaryDirectory() as tmpdir:
            db_filepath = os.path.join(tmpdir, "temp.db")
            with MonitoringDatabase(db_filepath) as db:
                update_events = [
                    ParseDict(
                        js_dict=dict(
                            event_type=ADD,
                            cpu=dict(
                                usage=0.1,
                            ),
                        ),
                        message=UpdateEvent(),
                    ),
                ]
                request = DataUpdateRequest(
                    machine_id=1,
                    machine_name="machine A",
                    timestamp=1000,
                    events=update_events,
                )
                db.store_data_update_request(request)

                messages = await sql_cmd(
                    db_filepath,
                    "SELECT * FROM CPU",
                )
                self.assertEqual(len(messages), 1)
                self.assertIn("usage", messages[0])
                self.assertIn("0.1", messages[0])



import unittest
from unittest import mock

from ...utils.testing import async_test
from .run_notifiers import DISCORD_MSG_LIMIT, MSG_TOO_LONG, log_and_send_msg_if_any


class TestRunNotifiers(unittest.TestCase):

    # pylint: disable=no-self-use
    @async_test
    async def test_msg_is_sent(self):
        channel_mock = mock.AsyncMock()
        channel_mock.send.return_value = None
        logger_mock = mock.MagicMock()
        logger_mock.info.return_value = None

        messages = [
            "This is the first message",
        ]

        await log_and_send_msg_if_any(
            messages=messages, logger=logger_mock, channel=channel_mock, is_testing=False
        )

        calls = [mock.call(msg) for msg in messages]
        channel_mock.send.assert_has_calls(calls)
        logger_mock.info.assert_has_calls(calls)

    # pylint: disable=no-self-use
    @async_test
    async def test_log_and_send_msg_if_any_msg_too_long(self):
        channel_mock = mock.AsyncMock()
        channel_mock.send.return_value = None
        logger_mock = mock.MagicMock()
        logger_mock.info.return_value = None

        messages = [
            "A" * (DISCORD_MSG_LIMIT + 1),
        ]

        await log_and_send_msg_if_any(
            messages=messages, logger=logger_mock, channel=channel_mock, is_testing=False
        )

        msg = MSG_TOO_LONG.format(n_chars_too_long=1, discord_msg_limit=DISCORD_MSG_LIMIT)
        channel_mock.send.assert_called_once_with(msg)
        logger_mock.info.assert_called_once_with(msg)

    # pylint: disable=no-self-use
    @async_test
    async def test_msg_is_not_sent_to_discord_during_testing(self):
        channel_mock = mock.AsyncMock()
        channel_mock.send.return_value = None
        logger_mock = mock.MagicMock()
        logger_mock.info.return_value = None

        messages = ["Some message"]

        await log_and_send_msg_if_any(
            messages=messages, logger=logger_mock, channel=channel_mock, is_testing=True
        )

        channel_mock.send.assert_not_called()
        logger_mock.info.assert_called_once_with(messages[0])

    # pylint: disable=no-self-use
    @async_test
    async def test_multiple_messages_are_bundled(self):
        channel_mock = mock.AsyncMock()
        channel_mock.send.return_value = None
        logger_mock = mock.MagicMock()
        logger_mock.info.return_value = None

        messages = ["This is the first message", "This is the second message"]
        total_msg = "\n".join(messages)

        await log_and_send_msg_if_any(
            messages=messages, logger=logger_mock, channel=channel_mock, is_testing=False
        )

        channel_mock.send.assert_called_once_with(total_msg)
        logger_mock.info.assert_called_once_with(total_msg)

    # pylint: disable=no-self-use
    @async_test
    async def test_multiple_messages_are_sent_individually_if_too_long(self):
        channel_mock = mock.AsyncMock()
        channel_mock.send.return_value = None
        logger_mock = mock.MagicMock()
        logger_mock.info.return_value = None

        messages = ["A" * 1000, "A" * 1001]

        await log_and_send_msg_if_any(
            messages=messages, logger=logger_mock, channel=channel_mock, is_testing=False
        )

        calls = [mock.call(msg) for msg in messages]
        channel_mock.send.assert_has_calls(calls)
        logger_mock.info.assert_has_calls(calls)



import os
import tempfile
import unittest

from google.protobuf.json_format import ParseDict

from ...monitoring.MonitoringDatabase import MonitoringDatabase
from ...protobuf.generated.computer_info_pb2 import ADD, UpdateEvent
from ...protobuf.generated.monitoring_service_pb2 import DataUpdateRequest
from ...utils.testing import async_test
from .farmers import farmers_cmd


class TestFarmersCmd(unittest.TestCase):
    @async_test
    async def test_no_farmers_case(self) -> None:

        with tempfile.TemporaryDirectory() as tmpdir:
            db_filepath = os.path.join(tmpdir, "temp.db")

            with MonitoringDatabase(db_filepath):
                messages = await farmers_cmd(db_filepath)

                self.assertEqual(len(messages), 1)
                self.assertTrue(messages[0].startswith("No farmers"))

    @async_test
    async def test_not_running_farmers_not_displayed(self) -> None:

        with tempfile.TemporaryDirectory() as tmpdir:
            db_filepath = os.path.join(tmpdir, "tmp.db")

            with MonitoringDatabase(db_filepath) as db:
                update_event = ParseDict(
                    js_dict=dict(
                        event_type=ADD,
                        farmer=dict(
                            is_running=False,
                        ),
                    ),
                    message=UpdateEvent(),
                )
                request = DataUpdateRequest(
                    machine_id=2,
                    machine_name="machine B",
                    timestamp=1000,
                    events=[update_event],
                )
                db.store_data_update_request(request)

                messages = await farmers_cmd(db_filepath)

                self.assertEqual(len(messages), 1)
                self.assertTrue(messages[0].startswith("No farmers"))

    @async_test
    async def test_farmers_in_db_case(self) -> None:

        with tempfile.TemporaryDirectory() as tmpdir:
            db_filepath = os.path.join(tmpdir, "tmp.db")

            with MonitoringDatabase(db_filepath) as db:
                # machine A has an online farmer with one harvester
                update_events = [
                    ParseDict(
                        js_dict=dict(
                            event_type=ADD,
                            farmer=dict(
                                is_running=True,
                            ),
                        ),
                        message=UpdateEvent(),
                    ),
                    ParseDict(
                        js_dict=dict(
                            event_type=ADD,
                            farmer_harvester=dict(
                                connection_time=500,
                                time_last_msg_received=999,
                                time_last_msg_sent=998,
                                ip_address="1.2.3.4",
                                missed_challenges=1,
                                n_plots=784,
                            ),
                        ),
                        message=UpdateEvent(),
                    ),
                ]
                request = DataUpdateRequest(
                    machine_id=1,
                    machine_name="machine A",
                    timestamp=1000,
                    events=update_events,
                )
                db.store_data_update_request(request)

                messages = await farmers_cmd(db_filepath)

                # no empty message
                self.assertGreater(len(messages), 1)
                self.assertFalse(messages[0].startswith("No farmers"))
                # display online harvester
                self.assertTrue(any("machine A" in msg for msg in messages))
                self.assertEqual(sum("Harvester" in msg for msg in messages), 1)
                # offline harvesters must not be displayed
                self.assertTrue(all("machine B" not in msg for msg in messages))



import asyncio
import os
from typing import AsyncGenerator, Awaitable, Callable, Coroutine, Optional, TextIO, Union

from ...utils.logger import get_logger


async def watch_lines_infinitely(
    filepath: str,
    on_file_missing: Optional[Coroutine] = None,
    on_ready: Optional[Coroutine] = None,
    on_line: Optional[Callable[[str], Awaitable[None]]] = None,
):
    """Start watching the specified file

    Parameters
    ----------
    filepath : str
        filepath to the file to be watched
    on_file_missing: Optional[Coroutine]
        function to be called when the specified logfile
        does not exist
    on_ready : Optional[Coroutine]
        function to be called when the initial startup
        has been completed (all current lines read)
    on_line : Optional[Callable[[str], Awaitable[None]]]
        function to be triggered if a new line has
        been found in the file
    """

    if not filepath:
        if on_file_missing is not None:
            await on_file_missing()
        return

    logger = get_logger(__name__)
    logger.debug("Searching logfile: %s", filepath)

    # try to watch file
    file_missing_was_run = False
    line_generator: Union[None, AsyncGenerator[str, None]] = None
    while line_generator is None:
        try:
            line_generator = watch_logfile_generator(
                filepath=filepath,
                on_ready=on_ready,
            )
        except FileNotFoundError:
            # in case there is no log file (yet) simply
            # wait gently for one to appear
            logger.info("Logfile %s not found, waiting for one to appear.", filepath)
            await asyncio.sleep(3)
            if on_file_missing is not None and not file_missing_was_run:
                await on_file_missing()
                file_missing_was_run = True

    logger.debug("Logfile '%s' found. Starting to watch it.", filepath)

    # process lines
    async for line in line_generator:
        if on_line is not None:
            await on_line(line)


async def watch_logfile_generator(
    filepath: str, on_ready: Optional[Coroutine] = None, interval_seconds: float = 1
) -> AsyncGenerator[str, None]:
    """Watch a logfile for changes

    Parameters
    ----------
    filepath : str
        path to the logfile to watch
    on_ready: Optional[Coroutine] = None,
        function to be called if initial lines were loaded
    interval_seconds : str
        timing interval in which to check the file for new content

    Yields
    ------
    line : str
        a newly added line to the file
    """
    logger = get_logger(__name__)

    if filepath.startswith("~"):
        filepath = os.path.expanduser(filepath)

    # loop for reopening the log file
    # chia uses a rotating logging scheme
    # moving the log file when it's full
    # and making a new one in-place.
    # To keep track we need to reopen the new file
    while True:
        logger.debug("(Re)opening logfile: %s", filepath)
        with open(filepath, "r", encoding="utf8") as fp:
            while True:

                # yield as many lines as there are
                new_line = fp.readline()
                while new_line:

                    # must be placed here so when we yielded
                    # the last line we caught up
                    if _end_of_file(fp):
                        await on_ready()

                    terminate = yield new_line
                    if terminate:
                        raise StopAsyncIteration()
                    new_line = fp.readline()

                # sleep to give it a rest
                await asyncio.sleep(interval_seconds)

                # check if file was replaced, then rewind
                if _file_was_replaced_or_cleared(fp, filepath):
                    break

        if terminate:
            break


def _end_of_file(fp: TextIO) -> bool:
    return fp.tell() == os.stat(fp.fileno()).st_size


def _file_was_replaced_or_cleared(fp: TextIO, filepath: str) -> bool:
    return fp.tell() > os.stat(filepath).st_size



import sqlite3
from typing import Union

from google.protobuf.json_format import MessageToDict

from ..protobuf.generated.computer_info_pb2 import ComputerInfo
from ..protobuf.generated.machine_info_pb2 import MachineInfo
from ..protobuf.generated.monitoring_service_pb2 import DataUpdateRequest
from ..protobuf.to_sqlite.sql_cmds import (
    ALL_SQL_CREATE_TABLE_CMDS,
    get_computer_info_from_db,
    insert_machine_info_in_db,
    insert_update_event_in_db,
    update_state_tables_in_db,
)
from ..utils.logger import get_logger


class MonitoringDatabase:
    """Used to store and retrieve data from the monitoring database"""

    filepath: str
    connection: Union[sqlite3.Connection, None]
    cursor: Union[sqlite3.Cursor, None]

    def __init__(self, filepath: str):
        self.filepath = filepath
        self.connection = None
        self.cursor = None

    def __check_if_initialized(self):
        if self.connection is None or self.cursor is None:
            err_msg = "MonitoringDatabase needs to be initialized by a 'with' statement in python."
            raise ValueError(err_msg)

    def __init_tables(self):
        logger = get_logger(__name__)

        logger.debug("Initializing database tables")
        self.__check_if_initialized()

        for cmd in ALL_SQL_CREATE_TABLE_CMDS:
            logger.debug(cmd)
            self.cursor.execute(cmd)

        logger.debug("Database Initialized")

    def __enter__(self):
        self.connection = sqlite3.connect(self.filepath)
        self.cursor = self.connection.cursor()

        self.__init_tables()

        self.connection.commit()

        return self

    def __exit__(self, exc_type, exc_val, exc_tb):

        # we don't handle the exception arguments since
        # this needs to be done further up. But they
        # need to exist to have a correct function
        # signature.
        #
        # See: https://docs.python.org/3/reference/datamodel.html#object.__exit__

        self.connection.commit()
        self.connection.close()

        self.cursor = None
        self.connection = None

    def get_machine_state(self, machine_id: str) -> ComputerInfo:
        """Get the state of a machine

        Parameters
        ----------
        machine_id : str
            machine to get state of

        Returns
        -------
        computer_info : ComputerInfo
            latest information about the machine
        """

        computer_info = get_computer_info_from_db(
            self.cursor,
            machine_id,
        )

        return computer_info

    def store_data_update_request(
        self,
        data_update_request: DataUpdateRequest,
        ip_address: str = "",
    ):
        """Store the data in an update request in the database

        Parameters
        ----------
        data_update_request : DataUpdateRequest
            data update request to store update events
        ip_address : str
            ip address of the remote machine
        """
        self.__check_if_initialized()

        logger = get_logger(__file__)

        # store info that the db was updated
        insert_machine_info_in_db(
            self.cursor,
            MachineInfo(
                machine_id=data_update_request.machine_id,
                name=data_update_request.machine_name,
                ip_address=ip_address,
                time_last_msg=data_update_request.timestamp,
            ),
            {},
        )

        for event in data_update_request.events:
            logger.debug(
                "Received event: %s",
                MessageToDict(event),
            )

            insert_update_event_in_db(
                sql_cursor=self.cursor,
                pb_message=event,
                meta_attributes=dict(
                    machine_id=data_update_request.machine_id,
                    timestamp=data_update_request.timestamp,
                    event_type=event.event_type,
                ),
            )

            update_state_tables_in_db(
                sql_cursor=self.cursor,
                pb_message=event,
                meta_attributes=dict(
                    machine_id=data_update_request.machine_id,
                ),
                event_type=event.event_type,
            )

        self.connection.commit()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chia_tea/protobuf/generated/config.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='chia_tea/protobuf/generated/config.proto',
  package='chia_tea.protobuf.generated.config_pb2',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n(chia_tea/protobuf/generated/config.proto\x12&chia_tea.protobuf.generated.config_pb2\"\x1d\n\rMachineConfig\x12\x0c\n\x04name\x18\x01 \x01(\t\"\xb3\x01\n\rLoggingConfig\x12\x42\n\x08loglevel\x18\x01 \x01(\x0e\x32\x30.chia_tea.protobuf.generated.config_pb2.LogLevel\x12\x16\n\x0elog_to_console\x18\x02 \x01(\x08\x12\x13\n\x0blog_to_file\x18\x03 \x01(\x08\x12\x14\n\x0cmax_logfiles\x18\x04 \x01(\x05\x12\x1b\n\x13max_logfile_size_mb\x18\x05 \x01(\x05\"<\n\nCopyConfig\x12\x16\n\x0esource_folders\x18\x01 \x03(\t\x12\x16\n\x0etarget_folders\x18\x02 \x03(\t\">\n\nChiaConfig\x12\x18\n\x10logfile_filepath\x18\x01 \x01(\t\x12\x16\n\x0emadmax_logfile\x18\x02 \x01(\t\"2\n\rDiscordConfig\x12\r\n\x05token\x18\x01 \x01(\t\x12\x12\n\nchannel_id\x18\x02 \x01(\x03\"\x88\x06\n\x10MonitoringConfig\x12Q\n\x04\x61uth\x18\x01 \x01(\x0b\x32\x43.chia_tea.protobuf.generated.config_pb2.MonitoringConfig.AuthConfig\x12U\n\x06server\x18\x02 \x01(\x0b\x32\x45.chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ServerConfig\x12U\n\x06\x63lient\x18\x03 \x01(\x0b\x32\x45.chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig\x1a\x39\n\nAuthConfig\x12\x15\n\rcert_filepath\x18\x01 \x01(\t\x12\x14\n\x0ckey_filepath\x18\x02 \x01(\t\x1a\x31\n\x0cServerConfig\x12\x0c\n\x04port\x18\x01 \x01(\x05\x12\x13\n\x0b\x64\x62_filepath\x18\x02 \x01(\t\x1a\x84\x03\n\x0c\x43lientConfig\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\x12\x1a\n\x12\x63ollect_data_every\x18\x03 \x01(\x01\x12p\n\x11send_update_every\x18\x04 \x01(\x0b\x32U.chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.SendUpdateEvery\x1a\xc6\x01\n\x0fSendUpdateEvery\x12\x0b\n\x03\x63pu\x18\x01 \x01(\x01\x12\x0b\n\x03ram\x18\x02 \x01(\x01\x12\x0c\n\x04\x64isk\x18\x03 \x01(\x01\x12\x0f\n\x07process\x18\x04 \x01(\x01\x12\x0e\n\x06\x66\x61rmer\x18\x05 \x01(\x01\x12\x18\n\x10\x66\x61rmer_harvester\x18\x06 \x01(\x01\x12\x11\n\tharvester\x18\x07 \x01(\x01\x12\x0e\n\x06wallet\x18\x08 \x01(\x01\x12\x15\n\rplotting_plot\x18\t \x01(\x01\x12\x16\n\x0eharvester_plot\x18\n \x01(\x01\"J\n\x11\x44\x65velopmentConfig\x12\x0f\n\x07testing\x18\x01 \x01(\x08\x12$\n\x1cmonitoring_client_state_file\x18\x02 \x01(\t\"\x9a\x04\n\rChiaTeaConfig\x12\x0f\n\x07version\x18\x01 \x01(\x05\x12\x46\n\x07machine\x18\x08 \x01(\x0b\x32\x35.chia_tea.protobuf.generated.config_pb2.MachineConfig\x12\x46\n\x07logging\x18\x02 \x01(\x0b\x32\x35.chia_tea.protobuf.generated.config_pb2.LoggingConfig\x12@\n\x04\x63opy\x18\x03 \x01(\x0b\x32\x32.chia_tea.protobuf.generated.config_pb2.CopyConfig\x12@\n\x04\x63hia\x18\x04 \x01(\x0b\x32\x32.chia_tea.protobuf.generated.config_pb2.ChiaConfig\x12\x46\n\x07\x64iscord\x18\x05 \x01(\x0b\x32\x35.chia_tea.protobuf.generated.config_pb2.DiscordConfig\x12L\n\nmonitoring\x18\x06 \x01(\x0b\x32\x38.chia_tea.protobuf.generated.config_pb2.MonitoringConfig\x12N\n\x0b\x64\x65velopment\x18\x07 \x01(\x0b\x32\x39.chia_tea.protobuf.generated.config_pb2.DevelopmentConfig*B\n\x08LogLevel\x12\t\n\x05TRACE\x10\x00\x12\t\n\x05\x44\x45\x42UG\x10\x01\x12\x08\n\x04INFO\x10\x02\x12\x0b\n\x07WARNING\x10\x03\x12\t\n\x05\x45RROR\x10\x04\x62\x06proto3'
)

_LOGLEVEL = _descriptor.EnumDescriptor(
  name='LogLevel',
  full_name='chia_tea.protobuf.generated.config_pb2.LogLevel',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TRACE', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DEBUG', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INFO', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='WARNING', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1871,
  serialized_end=1937,
)
_sym_db.RegisterEnumDescriptor(_LOGLEVEL)

LogLevel = enum_type_wrapper.EnumTypeWrapper(_LOGLEVEL)
TRACE = 0
DEBUG = 1
INFO = 2
WARNING = 3
ERROR = 4



_MACHINECONFIG = _descriptor.Descriptor(
  name='MachineConfig',
  full_name='chia_tea.protobuf.generated.config_pb2.MachineConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='chia_tea.protobuf.generated.config_pb2.MachineConfig.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=84,
  serialized_end=113,
)


_LOGGINGCONFIG = _descriptor.Descriptor(
  name='LoggingConfig',
  full_name='chia_tea.protobuf.generated.config_pb2.LoggingConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='loglevel', full_name='chia_tea.protobuf.generated.config_pb2.LoggingConfig.loglevel', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='log_to_console', full_name='chia_tea.protobuf.generated.config_pb2.LoggingConfig.log_to_console', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='log_to_file', full_name='chia_tea.protobuf.generated.config_pb2.LoggingConfig.log_to_file', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_logfiles', full_name='chia_tea.protobuf.generated.config_pb2.LoggingConfig.max_logfiles', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_logfile_size_mb', full_name='chia_tea.protobuf.generated.config_pb2.LoggingConfig.max_logfile_size_mb', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=116,
  serialized_end=295,
)


_COPYCONFIG = _descriptor.Descriptor(
  name='CopyConfig',
  full_name='chia_tea.protobuf.generated.config_pb2.CopyConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='source_folders', full_name='chia_tea.protobuf.generated.config_pb2.CopyConfig.source_folders', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='target_folders', full_name='chia_tea.protobuf.generated.config_pb2.CopyConfig.target_folders', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=297,
  serialized_end=357,
)


_CHIACONFIG = _descriptor.Descriptor(
  name='ChiaConfig',
  full_name='chia_tea.protobuf.generated.config_pb2.ChiaConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='logfile_filepath', full_name='chia_tea.protobuf.generated.config_pb2.ChiaConfig.logfile_filepath', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='madmax_logfile', full_name='chia_tea.protobuf.generated.config_pb2.ChiaConfig.madmax_logfile', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=359,
  serialized_end=421,
)


_DISCORDCONFIG = _descriptor.Descriptor(
  name='DiscordConfig',
  full_name='chia_tea.protobuf.generated.config_pb2.DiscordConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='token', full_name='chia_tea.protobuf.generated.config_pb2.DiscordConfig.token', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='channel_id', full_name='chia_tea.protobuf.generated.config_pb2.DiscordConfig.channel_id', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=423,
  serialized_end=473,
)


_MONITORINGCONFIG_AUTHCONFIG = _descriptor.Descriptor(
  name='AuthConfig',
  full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.AuthConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='cert_filepath', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.AuthConfig.cert_filepath', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='key_filepath', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.AuthConfig.key_filepath', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=753,
  serialized_end=810,
)

_MONITORINGCONFIG_SERVERCONFIG = _descriptor.Descriptor(
  name='ServerConfig',
  full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ServerConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='port', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ServerConfig.port', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='db_filepath', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ServerConfig.db_filepath', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=812,
  serialized_end=861,
)

_MONITORINGCONFIG_CLIENTCONFIG_SENDUPDATEEVERY = _descriptor.Descriptor(
  name='SendUpdateEvery',
  full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.SendUpdateEvery',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='cpu', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.SendUpdateEvery.cpu', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ram', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.SendUpdateEvery.ram', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='disk', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.SendUpdateEvery.disk', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='process', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.SendUpdateEvery.process', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='farmer', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.SendUpdateEvery.farmer', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='farmer_harvester', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.SendUpdateEvery.farmer_harvester', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='harvester', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.SendUpdateEvery.harvester', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='wallet', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.SendUpdateEvery.wallet', index=7,
      number=8, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='plotting_plot', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.SendUpdateEvery.plotting_plot', index=8,
      number=9, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='harvester_plot', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.SendUpdateEvery.harvester_plot', index=9,
      number=10, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1054,
  serialized_end=1252,
)

_MONITORINGCONFIG_CLIENTCONFIG = _descriptor.Descriptor(
  name='ClientConfig',
  full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='address', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.address', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='port', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.port', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='collect_data_every', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.collect_data_every', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='send_update_every', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.send_update_every', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_MONITORINGCONFIG_CLIENTCONFIG_SENDUPDATEEVERY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=864,
  serialized_end=1252,
)

_MONITORINGCONFIG = _descriptor.Descriptor(
  name='MonitoringConfig',
  full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='auth', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.auth', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='server', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.server', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='client', full_name='chia_tea.protobuf.generated.config_pb2.MonitoringConfig.client', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_MONITORINGCONFIG_AUTHCONFIG, _MONITORINGCONFIG_SERVERCONFIG, _MONITORINGCONFIG_CLIENTCONFIG, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=476,
  serialized_end=1252,
)


_DEVELOPMENTCONFIG = _descriptor.Descriptor(
  name='DevelopmentConfig',
  full_name='chia_tea.protobuf.generated.config_pb2.DevelopmentConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='testing', full_name='chia_tea.protobuf.generated.config_pb2.DevelopmentConfig.testing', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='monitoring_client_state_file', full_name='chia_tea.protobuf.generated.config_pb2.DevelopmentConfig.monitoring_client_state_file', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1254,
  serialized_end=1328,
)


_CHIATEACONFIG = _descriptor.Descriptor(
  name='ChiaTeaConfig',
  full_name='chia_tea.protobuf.generated.config_pb2.ChiaTeaConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='chia_tea.protobuf.generated.config_pb2.ChiaTeaConfig.version', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='machine', full_name='chia_tea.protobuf.generated.config_pb2.ChiaTeaConfig.machine', index=1,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='logging', full_name='chia_tea.protobuf.generated.config_pb2.ChiaTeaConfig.logging', index=2,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='copy', full_name='chia_tea.protobuf.generated.config_pb2.ChiaTeaConfig.copy', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='chia', full_name='chia_tea.protobuf.generated.config_pb2.ChiaTeaConfig.chia', index=4,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='discord', full_name='chia_tea.protobuf.generated.config_pb2.ChiaTeaConfig.discord', index=5,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='monitoring', full_name='chia_tea.protobuf.generated.config_pb2.ChiaTeaConfig.monitoring', index=6,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='development', full_name='chia_tea.protobuf.generated.config_pb2.ChiaTeaConfig.development', index=7,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1331,
  serialized_end=1869,
)

_LOGGINGCONFIG.fields_by_name['loglevel'].enum_type = _LOGLEVEL
_MONITORINGCONFIG_AUTHCONFIG.containing_type = _MONITORINGCONFIG
_MONITORINGCONFIG_SERVERCONFIG.containing_type = _MONITORINGCONFIG
_MONITORINGCONFIG_CLIENTCONFIG_SENDUPDATEEVERY.containing_type = _MONITORINGCONFIG_CLIENTCONFIG
_MONITORINGCONFIG_CLIENTCONFIG.fields_by_name['send_update_every'].message_type = _MONITORINGCONFIG_CLIENTCONFIG_SENDUPDATEEVERY
_MONITORINGCONFIG_CLIENTCONFIG.containing_type = _MONITORINGCONFIG
_MONITORINGCONFIG.fields_by_name['auth'].message_type = _MONITORINGCONFIG_AUTHCONFIG
_MONITORINGCONFIG.fields_by_name['server'].message_type = _MONITORINGCONFIG_SERVERCONFIG
_MONITORINGCONFIG.fields_by_name['client'].message_type = _MONITORINGCONFIG_CLIENTCONFIG
_CHIATEACONFIG.fields_by_name['machine'].message_type = _MACHINECONFIG
_CHIATEACONFIG.fields_by_name['logging'].message_type = _LOGGINGCONFIG
_CHIATEACONFIG.fields_by_name['copy'].message_type = _COPYCONFIG
_CHIATEACONFIG.fields_by_name['chia'].message_type = _CHIACONFIG
_CHIATEACONFIG.fields_by_name['discord'].message_type = _DISCORDCONFIG
_CHIATEACONFIG.fields_by_name['monitoring'].message_type = _MONITORINGCONFIG
_CHIATEACONFIG.fields_by_name['development'].message_type = _DEVELOPMENTCONFIG
DESCRIPTOR.message_types_by_name['MachineConfig'] = _MACHINECONFIG
DESCRIPTOR.message_types_by_name['LoggingConfig'] = _LOGGINGCONFIG
DESCRIPTOR.message_types_by_name['CopyConfig'] = _COPYCONFIG
DESCRIPTOR.message_types_by_name['ChiaConfig'] = _CHIACONFIG
DESCRIPTOR.message_types_by_name['DiscordConfig'] = _DISCORDCONFIG
DESCRIPTOR.message_types_by_name['MonitoringConfig'] = _MONITORINGCONFIG
DESCRIPTOR.message_types_by_name['DevelopmentConfig'] = _DEVELOPMENTCONFIG
DESCRIPTOR.message_types_by_name['ChiaTeaConfig'] = _CHIATEACONFIG
DESCRIPTOR.enum_types_by_name['LogLevel'] = _LOGLEVEL
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MachineConfig = _reflection.GeneratedProtocolMessageType('MachineConfig', (_message.Message,), {
  'DESCRIPTOR' : _MACHINECONFIG,
  '__module__' : 'chia_tea.protobuf.generated.config_pb2'
  # @@protoc_insertion_point(class_scope:chia_tea.protobuf.generated.config_pb2.MachineConfig)
  })
_sym_db.RegisterMessage(MachineConfig)

LoggingConfig = _reflection.GeneratedProtocolMessageType('LoggingConfig', (_message.Message,), {
  'DESCRIPTOR' : _LOGGINGCONFIG,
  '__module__' : 'chia_tea.protobuf.generated.config_pb2'
  # @@protoc_insertion_point(class_scope:chia_tea.protobuf.generated.config_pb2.LoggingConfig)
  })
_sym_db.RegisterMessage(LoggingConfig)

CopyConfig = _reflection.GeneratedProtocolMessageType('CopyConfig', (_message.Message,), {
  'DESCRIPTOR' : _COPYCONFIG,
  '__module__' : 'chia_tea.protobuf.generated.config_pb2'
  # @@protoc_insertion_point(class_scope:chia_tea.protobuf.generated.config_pb2.CopyConfig)
  })
_sym_db.RegisterMessage(CopyConfig)

ChiaConfig = _reflection.GeneratedProtocolMessageType('ChiaConfig', (_message.Message,), {
  'DESCRIPTOR' : _CHIACONFIG,
  '__module__' : 'chia_tea.protobuf.generated.config_pb2'
  # @@protoc_insertion_point(class_scope:chia_tea.protobuf.generated.config_pb2.ChiaConfig)
  })
_sym_db.RegisterMessage(ChiaConfig)

DiscordConfig = _reflection.GeneratedProtocolMessageType('DiscordConfig', (_message.Message,), {
  'DESCRIPTOR' : _DISCORDCONFIG,
  '__module__' : 'chia_tea.protobuf.generated.config_pb2'
  # @@protoc_insertion_point(class_scope:chia_tea.protobuf.generated.config_pb2.DiscordConfig)
  })
_sym_db.RegisterMessage(DiscordConfig)

MonitoringConfig = _reflection.GeneratedProtocolMessageType('MonitoringConfig', (_message.Message,), {

  'AuthConfig' : _reflection.GeneratedProtocolMessageType('AuthConfig', (_message.Message,), {
    'DESCRIPTOR' : _MONITORINGCONFIG_AUTHCONFIG,
    '__module__' : 'chia_tea.protobuf.generated.config_pb2'
    # @@protoc_insertion_point(class_scope:chia_tea.protobuf.generated.config_pb2.MonitoringConfig.AuthConfig)
    })
  ,

  'ServerConfig' : _reflection.GeneratedProtocolMessageType('ServerConfig', (_message.Message,), {
    'DESCRIPTOR' : _MONITORINGCONFIG_SERVERCONFIG,
    '__module__' : 'chia_tea.protobuf.generated.config_pb2'
    # @@protoc_insertion_point(class_scope:chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ServerConfig)
    })
  ,

  'ClientConfig' : _reflection.GeneratedProtocolMessageType('ClientConfig', (_message.Message,), {

    'SendUpdateEvery' : _reflection.GeneratedProtocolMessageType('SendUpdateEvery', (_message.Message,), {
      'DESCRIPTOR' : _MONITORINGCONFIG_CLIENTCONFIG_SENDUPDATEEVERY,
      '__module__' : 'chia_tea.protobuf.generated.config_pb2'
      # @@protoc_insertion_point(class_scope:chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig.SendUpdateEvery)
      })
    ,
    'DESCRIPTOR' : _MONITORINGCONFIG_CLIENTCONFIG,
    '__module__' : 'chia_tea.protobuf.generated.config_pb2'
    # @@protoc_insertion_point(class_scope:chia_tea.protobuf.generated.config_pb2.MonitoringConfig.ClientConfig)
    })
  ,
  'DESCRIPTOR' : _MONITORINGCONFIG,
  '__module__' : 'chia_tea.protobuf.generated.config_pb2'
  # @@protoc_insertion_point(class_scope:chia_tea.protobuf.generated.config_pb2.MonitoringConfig)
  })
_sym_db.RegisterMessage(MonitoringConfig)
_sym_db.RegisterMessage(MonitoringConfig.AuthConfig)
_sym_db.RegisterMessage(MonitoringConfig.ServerConfig)
_sym_db.RegisterMessage(MonitoringConfig.ClientConfig)
_sym_db.RegisterMessage(MonitoringConfig.ClientConfig.SendUpdateEvery)

DevelopmentConfig = _reflection.GeneratedProtocolMessageType('DevelopmentConfig', (_message.Message,), {
  'DESCRIPTOR' : _DEVELOPMENTCONFIG,
  '__module__' : 'chia_tea.protobuf.generated.config_pb2'
  # @@protoc_insertion_point(class_scope:chia_tea.protobuf.generated.config_pb2.DevelopmentConfig)
  })
_sym_db.RegisterMessage(DevelopmentConfig)

ChiaTeaConfig = _reflection.GeneratedProtocolMessageType('ChiaTeaConfig', (_message.Message,), {
  'DESCRIPTOR' : _CHIATEACONFIG,
  '__module__' : 'chia_tea.protobuf.generated.config_pb2'
  # @@protoc_insertion_point(class_scope:chia_tea.protobuf.generated.config_pb2.ChiaTeaConfig)
  })
_sym_db.RegisterMessage(ChiaTeaConfig)


# @@protoc_insertion_point(module_scope)



import asyncio
import traceback
from typing import Union

import grpc
from google.protobuf.json_format import MessageToDict

from ..protobuf.generated.monitoring_service_pb2 import DataUpdateRequest
from ..protobuf.generated.monitoring_service_pb2_grpc import MonitoringServicer
from ..utils.logger import get_logger
from .MonitoringDatabase import MonitoringDatabase


class MonitoringServer(MonitoringServicer):
    """This class contains the server receiving and storing
    monitoring data from monitoring clients.
    """

    db: MonitoringDatabase

    def __init__(self, db: MonitoringDatabase):
        """Constructor

        Parameters
        ----------
        db : MonitoringDatabase
            database to operate on
        """
        super().__init__()
        self.db = db

    # pylint: disable=invalid-overridden-method
    async def GetMachineState(self, request, context):
        """Stub to get the latest state of a machine"""
        logger = get_logger(__name__)

        msg = "Received message from {address}: {message}".format(
            address=context.peer(),
            message=MessageToDict(request),
        )
        logger.info(msg)

        return self.db.get_machine_state(request.machine_id)

    # pylint: disable=invalid-overridden-method
    async def SendMonitoringUpdate(self, request_iterator, context):
        """Stub to receive monitoring updates from clients"""
        logger = get_logger(__name__)
        logger.info("Connected to %s", context.peer())

        # we endlessly process updates
        while True:
            try:
                # check for a response
                data_update_request: Union[DataUpdateRequest, grpc.aio.EOF] = await context.read()

                # last stream ended (batch of messages)
                if data_update_request == grpc.aio.EOF:
                    continue

                logger.info("Received update from %s", context.peer())

                if not data_update_request.timestamp:
                    raise ValueError("DataUpdateRequest requires a timestamp.")

                if not data_update_request.machine_id:
                    raise ValueError("DataUpdateRequest requires a machine id.")

                # store in database
                self.db.store_data_update_request(
                    data_update_request=data_update_request, ip_address=context.peer()
                )

            except asyncio.TimeoutError:
                # try again
                pass
            except Exception:
                trace = traceback.format_exc()
                logger.error(trace)
                await asyncio.sleep(5)



import argparse
import os
import sys

from ..utils.cli import str2bool
from ..utils.config import get_default_config, read_config, save_config
from ..utils.logger import get_logger

module_name = "chia_tea.config"


def _parse_args() -> argparse.Namespace:
    """Parse the arguments from the command line

    Returns
    -------
    args : argparse.Namespace
        parsed arguments
    """

    parser = argparse.ArgumentParser(
        prog="Chia-Tea Config",
        description=(
            "This tool is used to either create a default config"
            + " or validate an existing config."
        ),
    )

    parser.add_argument("config", help="Path to the config file")
    parser.add_argument(
        "--generate",
        type=str2bool,
        nargs="?",
        default=False,
        const=True,
        help="Generate a default config",
    )
    parser.add_argument(
        "--validate",
        type=str2bool,
        nargs="?",
        default=False,
        const=True,
        help="Generate a default config",
    )
    parser.add_argument(
        "--overwrite",
        type=str2bool,
        nargs="?",
        default=False,
        const=True,
        help="Generate a default config",
    )

    if len(sys.argv) < 1:
        parser.print_help()
        sys.exit(0)

    return parser.parse_args(sys.argv[1:])


def _generate_default_config(
    filepath: str,
    overwrite: bool,
):
    """Generate a default config on the specified path

    Parameters
    ----------
    filepath : str
        path where to create the default config
    overwrite : bool
        whether to overwrite in case the file already exists
    """
    logger = get_logger(module_name)

    if os.path.isfile(filepath) and not overwrite:
        err_msg = "Config file '%s' does already exist. Use '--overwrite' to overwrite this file."
        logger.error(err_msg, filepath)
        sys.exit(1)

    save_config(
        filepath=filepath,
        config=get_default_config(),
    )


def _validate_config(filepath: str):
    """Validate a chia-tea config

    Parameters
    ----------
    filepath : str
        path to the config file
    """
    logger = get_logger(module_name)

    try:
        read_config(filepath)
        logger.info("👍 Config '%s' is valid", filepath)
    except Exception as err:
        logger.error(str(err))
        sys.exit(1)


def main():
    """Main function for the config cli"""
    args = _parse_args()

    # generate a deffault config
    if args.generate:
        _generate_default_config(
            args.config,
            args.overwrite,
        )

    # validate the config
    if args.validate:
        _validate_config(args.config)


if __name__ == "__main__":
    main()



from dataclasses import dataclass


@dataclass
class FarmerHarvesterAPI:
    """A connected machine to the farmer

    schema of connections from example data
    {
        'bytes_read': 732920,
        'bytes_written': 736979,
        'creation_time': 1625781881.464225,
        'last_message_time': 1625856666.3932514,
        'local_port': 8447,
        'node_id': b'1n\x0f\xc4J\xb5q8\xc4\x98\x0b\xe7\\\xac\xd1\x82...',
        'peer_host': '127.0.0.1',
        'peer_port': 51844,
        'peer_server_port': 8448,
        'type': 2
    }
    """

    # pylint: disable=too-few-public-methods, too-many-instance-attributes

    node_id: bytes
    bytes_read: int
    bytes_written: int
    creation_time: float
    last_message_time: float
    local_port: int
    peer_host: str
    peer_port: int
    peer_server_port: int
    type: int

    # self-added extra attributes
    n_plots: int = 0



from dataclasses import dataclass
from datetime import datetime
from typing import Union


# pylint: disable=too-many-instance-attributes
@dataclass
class FarmerHarvesterLogfile:
    """Class with compact information about harvesters"""

    # General info
    harvester_id: str = ""
    ip_address: str = ""

    # Tracking of connection status
    is_connected = False

    # Signage point tracking
    time_of_end_of_last_sgn_point: Union[datetime, None] = None
    time_last_incoming_msg: Union[datetime, None] = None
    time_last_outgoing_msg: Union[datetime, None] = None
    timed_out: bool = False

    # Metrics
    n_responses = 0
    n_overdue_responses = 0

    # Additional tracking
    last_update: datetime = datetime.now()

    def check_for_timeout(self, current_time: datetime) -> None:
        """This functions checks if the harvester timed out from the view of a farmer

        Parameters
        ----------
        current_time : datetime
            current time to base computation on

        Notes
        -----
            Modifies internal attributes so don't spam.
        """
        CHALLENGE_TIMEOUT = 25  # seconds
        HARVESTER_TIMOUT = 60  # seconds

        if not self.timed_out:
            if self.time_last_incoming_msg is not None and self.time_last_outgoing_msg is not None:
                delta_seconds = (current_time - self.time_last_incoming_msg).total_seconds()
                if delta_seconds > CHALLENGE_TIMEOUT:
                    self.n_overdue_responses += 1
                if delta_seconds > HARVESTER_TIMOUT:
                    self.timed_out = True



import json
import os
import unittest

from google.protobuf.json_format import MessageToDict

from .config import get_default_config, read_config


class TestConfig(unittest.TestCase):
    def setUp(self) -> None:
        # pylint: disable=invalid-name
        self.maxDiff = None

    def test_default_config_matches_internal_default_config(self):
        internal_default_config = get_default_config()
        other_default_config = read_config(
            os.path.join(os.path.dirname(__file__), "..", "..", "config_default.yml")
        )

        self.assertEqual(
            json.dumps(MessageToDict(internal_default_config), indent=2),
            json.dumps(MessageToDict(other_default_config), indent=2),
        )



import asyncio

import grpc

from ..protobuf.generated.monitoring_service_pb2_grpc import (
    add_MonitoringServicer_to_server,
)
from ..protobuf.generated.config_pb2 import ChiaTeaConfig
from ..utils.cli import parse_args
from ..utils.config import read_config
from ..utils.logger import get_logger
from .common import get_credentials_cert, get_credentials_key
from .MonitoringDatabase import MonitoringDatabase
from .MonitoringServer import MonitoringServer


def __get_port(config: ChiaTeaConfig) -> int:
    """Get the port for the server

    Returns
    -------
    port : int
        port of the server from the config
    """
    port = config.monitoring.server.port

    if not isinstance(port, int):
        msg = "Monitoring server port '{0}' must be a positive number"
        raise ValueError(msg.format(port))

    if not port > 0:
        msg = "Monitoring server port '{0}' must be a positive number"
        raise ValueError(msg.format(port))

    return port


def __get_database_filepath(config: ChiaTeaConfig) -> str:
    """Get the database filepath from the config

    Parameters
    ---------
    config : ChiaTeaConfig
        config to read filepath for cert and key

    Returns
    -------
    db_filepath : str
        path where the database is or will be stored
    """
    filepath = config.monitoring.server.db_filepath

    if not filepath:
        raise ValueError("Config entry 'monitoring.server.db_filepath' may not be empty.")

    return filepath


def create_server(
    ip_address: str, cert: bytes, key: bytes, db: MonitoringDatabase
) -> grpc.aio.Server:
    """Creates a grpc server from the config"""
    logger = get_logger(__name__)

    server = grpc.aio.server()

    if cert and key:
        logger.info("Using encryption")
        server_credentials = grpc.ssl_server_credentials(
            [(key, cert)],
        )
        server.add_secure_port(ip_address, server_credentials)
    else:
        logger.warning("Encryption disabled")
        server.add_insecure_port(ip_address)

    add_MonitoringServicer_to_server(MonitoringServer(db=db), server)

    return server


async def build_server(config: ChiaTeaConfig, db: MonitoringDatabase):
    """Builds the monitoring server

    Parmeters
    ---------
    config : ChiaTeaConfig
        Config used by the server
    db : MonitoringDatabase
        database to store monitoring data in

    Returns
    -------
    server : grpc.aio.Server
        Monitoring server instance
    """

    # server address
    port = __get_port(config)
    ip_address = "[::]:" + str(port)

    # testing disables certificate security
    is_testing = config.development.testing

    # get certificates for security
    cert = get_credentials_cert(is_testing, config)
    key = get_credentials_key(is_testing, config)

    # build server
    server = create_server(ip_address, cert, key, db)

    return server


async def start_server(config: ChiaTeaConfig):
    """Builds and starts the server

    Parmeters
    ---------
    config : ChiaTeaConfig
        config used by the server
    """
    logger = get_logger(__name__)

    logger.debug("Initializing database")
    db_filepath = __get_database_filepath(config)
    with MonitoringDatabase(db_filepath) as db:

        logger.debug("Building server")
        server = await build_server(config, db)

        logger.info("Starting Server")
        await server.start()
        await server.wait_for_termination()


def main():
    """Main function starting the monitoring server"""

    try:
        args = parse_args(
            name="Chia Tea Monitoring Server",
            description=("Start a server collecting data from" + "from monitoring clients."),
        )

        # load config
        config = read_config(args.config)

        # setup logger
        logger = get_logger(__name__)

        # start the server
        loop = asyncio.get_event_loop()
        loop.run_until_complete(start_server(config))

    except KeyboardInterrupt:
        logger.info("Stopped server.")


if __name__ == "__main__":
    main()



from typing import Any, Dict, List, Set

from chia.rpc.farmer_rpc_client import FarmerRpcClient
from chia.server.outbound_message import NodeType
from chia.util.config import load_config
from chia.util.default_root import DEFAULT_ROOT_PATH
from chia.util.ints import uint16

from ..ChiaWatchdog import ChiaWatchdog
from .FarmerHarvesterAPI import FarmerHarvesterAPI
from .shared_settings import API_EXCEPTIONS

NODE_ID = "node_id"


async def _update_farmer_connections(
    farmer_client: FarmerRpcClient,
) -> List[FarmerHarvesterAPI]:

    harvesters = []

    # general connection information
    harvester_connections: Dict[str, Dict[str, Any]] = {
        data[NODE_ID].hex(): data
        for data in await farmer_client.get_connections()
        if data.get("type") == NodeType.HARVESTER.value
    }

    # harvester specific data such as plot count
    harvesters_which_disconnected: Set[str] = set()
    response = await farmer_client.get_harvesters()
    if response.get("success"):
        harvester_ids = set()
        harvester_data_list = response.get("harvesters")
        for data in harvester_data_list:
            node_id = data.get("connection").get(NODE_ID)
            harvester_ids.add(node_id)
            n_plots = len(data.get("plots"))
            properties = harvester_connections.get(node_id)

            if properties is not None:
                properties["n_plots"] = n_plots
            else:
                # A new harvester might have connected in-between.
                # We simply skip them.
                pass

        # harvesters which disconnected between the two API calls
        harvesters_which_disconnected = {
            node_id for node_id in harvester_connections.keys() if node_id not in harvester_ids
        }

    for node_id, kwargs in harvester_connections.items():

        # A harvester disconnected inbetween thus we skip it
        if node_id in harvesters_which_disconnected:
            continue

        harvesters.append(FarmerHarvesterAPI(**kwargs))

    return harvesters


async def update_from_farmer(chia_dog: ChiaWatchdog):
    """Updates the chia dog with harvester data

    Parameters
    ----------
    chia_dog : ChiaWatchdog
        watchdog instance to be modified
    """
    # pylint: disable=duplicate-code

    try:
        config = load_config(DEFAULT_ROOT_PATH, "config.yaml", exit_on_error=False)
        self_hostname = config["self_hostname"]
        farmer_rpc_port = config["farmer"]["rpc_port"]
        farmer_client = await FarmerRpcClient.create(
            self_hostname,
            uint16(farmer_rpc_port),
            DEFAULT_ROOT_PATH,
            config,
        )

        chia_dog.farmer_service.connections = await _update_farmer_connections(
            farmer_client=farmer_client,
        )
        chia_dog.farmer_service.is_running = True

    # pylint: disable=catching-non-exception
    except API_EXCEPTIONS:
        chia_dog.farmer_service.connections.clear()
        chia_dog.farmer_service.is_running = False
    finally:
        if "farmer_client" in locals():
            farmer_client.close()
            await farmer_client.await_closed()
        chia_dog.farmer_service.is_ready = True



from ..generated.computer_info_pb2 import (
    _UPDATEEVENT,
    ComputerInfo,
    EventType,
    UpdateEvent,
)
from ..generated.hardware_pb2 import Cpu, Ram
from ..generated.machine_info_pb2 import _MACHINEINFO, MachineInfo
from .custom import (
    SqliteType,
    get_event_table_insertion_cmds_for_nested_messages,
    get_fun_to_collect_latest_update_events_from_db,
    get_fun_to_collect_pb2_messages_for_nested_submessages,
    get_function_to_retrieve_pb2_from_sqlite_db,
    get_state_table_modification_fun_for_nested_messages,
    sqlite_create_event_tbl_cmd_from_pb2,
    sqlite_create_state_tbl_cmd_from_pb2,
    sqlite_insert_into_table_fun_from_pb2,
)
from .generic import ProtoType, get_create_table_cmds_for_enum

insert_update_event_in_db = get_event_table_insertion_cmds_for_nested_messages(
    _UPDATEEVENT
)

update_state_tables_in_db = get_state_table_modification_fun_for_nested_messages(
    _UPDATEEVENT
)

get_computer_info_from_db = get_fun_to_collect_pb2_messages_for_nested_submessages(
    pb_class=ComputerInfo,
    key_names_and_ops=[("machine_id", "=")],
    table_suffix="",
)

get_update_events_from_db = get_fun_to_collect_latest_update_events_from_db(
    pb_class=UpdateEvent,
    key_names_and_ops=[
        ("timestamp", ">="),
        ("timestamp", "<"),
    ],
    table_suffix="Events",
)

get_machine_infos_from_db = get_function_to_retrieve_pb2_from_sqlite_db(
    table_suffix="",
    pb_class=MachineInfo,
    key_names_and_ops=[],
)

get_cpu_for_machine_from_db = get_function_to_retrieve_pb2_from_sqlite_db(
    table_suffix="",
    pb_class=Cpu,
    key_names_and_ops=[("machine_id", "==")],
)

get_ram_for_machine_from_db = get_function_to_retrieve_pb2_from_sqlite_db(
    table_suffix="",
    pb_class=Ram,
    key_names_and_ops=[("machine_id", "==")],
)

insert_machine_info_in_db = sqlite_insert_into_table_fun_from_pb2(
    table_name=_MACHINEINFO.name,
    meta_attribute_names=[],
    pb_descriptor=_MACHINEINFO,
    fields_to_ignore=[],
)

SQL_CREATE_MACHINE_TBL_CMD = sqlite_create_state_tbl_cmd_from_pb2(
    _MACHINEINFO,
    meta_attributes=[],
    primary_key_names=[
        "machine_id",
    ],
    optional_primary_key_names=["id"],
)


ALL_SQL_CREATE_TABLE_CMDS = (
    # event tables
    tuple(
        sqlite_create_event_tbl_cmd_from_pb2(
            field.message_type,
            [
                ("timestamp", SqliteType.REAL),
                ("machine_id", SqliteType.INTEGER),
                ("event_type", SqliteType.INTEGER),
            ],
            primary_key_names=[
                "timestamp",
                "machine_id",
                # id is sometimes hacked in this is not nice ...
            ],
            optional_primary_key_names=["id"],
        )
        for field in _UPDATEEVENT.fields
        if field.type == ProtoType.MESSAGE.value
    )
    +
    # latest state tables
    tuple(
        sqlite_create_state_tbl_cmd_from_pb2(
            field.message_type,
            meta_attributes=[
                ("machine_id", SqliteType.INTEGER),
            ],
            primary_key_names=["machine_id"],
            optional_primary_key_names=["id"],
        )
        for field in _UPDATEEVENT.fields
        if field.type == ProtoType.MESSAGE.value
    )
    +
    # machine metadata table
    (SQL_CREATE_MACHINE_TBL_CMD,)
    +
    # enum tables
    tuple(get_create_table_cmds_for_enum(EventType))
)



import unittest

from ..protobuf.generated.chia_pb2 import HarvesterPlot
from ..protobuf.generated.computer_info_pb2 import (
    _COMPUTERINFO,
    ADD,
    DELETE,
    UPDATE,
    ComputerInfo,
    UpdateEvent,
)
from ..protobuf.generated.hardware_pb2 import Cpu
from ..utils.testing import async_test
from .computer_info_comparison import compare_computer_info


class TestMonitoringClient(unittest.TestCase):
    @async_test
    async def test_compare_empty_computer_info(self):

        old_computer_info = ComputerInfo()
        new_computer_info = ComputerInfo()
        events = [
            event async for event in compare_computer_info(old_computer_info, new_computer_info)
        ]
        self.assertListEqual(list(events), [])

    @async_test
    async def test_compare_computer_info_update_non_iterable(self):

        event_data = Cpu(name="my_cpu")

        old_computer_info = ComputerInfo()
        new_computer_info = ComputerInfo(cpu=event_data)
        events = [
            event async for event in compare_computer_info(old_computer_info, new_computer_info)
        ]
        expected = [
            UpdateEvent(
                event_type=UPDATE,
                cpu=event_data,
            )
        ]

        self.assertListEqual(list(events), expected)

    @async_test
    async def test_compare_computer_info_iterables(self):

        old_computer_info = ComputerInfo(
            harvester_plots=[
                HarvesterPlot(id="remove me"),
                HarvesterPlot(id="changed"),
                HarvesterPlot(id="nothing"),
            ]
        )
        new_computer_info = ComputerInfo(
            harvester_plots=[
                HarvesterPlot(
                    id="add me",
                ),
                HarvesterPlot(id="changed", filename="something changed"),
                HarvesterPlot(id="nothing"),
            ]
        )
        events = [
            event async for event in compare_computer_info(old_computer_info, new_computer_info)
        ]
        expected = [
            UpdateEvent(
                event_type=ADD,
                harvester_plot=HarvesterPlot(
                    id="add me",
                ),
            ),
            UpdateEvent(
                event_type=UPDATE,
                harvester_plot=HarvesterPlot(id="changed", filename="something changed"),
            ),
            UpdateEvent(
                event_type=DELETE,
                harvester_plot=HarvesterPlot(
                    id="remove me",
                ),
            ),
        ]

        self.assertListEqual(list(events), expected)

    def test_update_event_and_computer_info_have_matching_fields(self):

        computer_info_field_types = [
            field.message_type
            for field in _COMPUTERINFO.fields
            if field.name not in ("timestamp", "machine_id")
        ]
        update_event_type = [
            field.message_type for field in _COMPUTERINFO.fields if field.name != "event_type"
        ]

        for field_type1 in computer_info_field_types:
            self.assertIn(field_type1, update_event_type)
        # self.assertListEqual(computer_info_field_types, update_event_type)
